#-*-coding:utf8-*-
#############################################################################
#    Baram : Graphic User Interface for OpenFOAM
#    Copyright held by original author
#############################################################################
# License                                                                       
#    This file is part of Baram.                                            
#
#    Baram is free software; you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the
#    Free Software Foundation; either version 3 of the License, or (at your
#    option) any later version.
#
#    Baram is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#    for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Baram.  If not, see <http://www.gnu.org/licenses/>.
#
# Description
#    - Class for run incompressible flow solver
#
# Author
#    NEXTFOAM Inc. All rights reserved.
##############################################################################

import os,math,glob;import pygtk;pygtk.require('2.0');import gtk

from writeFileIncompressibleSub import *
from generalSub import *
from changeDict import *

class solvingClass:

    def delete_event(self,widget,event,data=None):
        print 'delete event occurred'
        return False

    def destroy(self,widget,data=None):
        self.Win.destroy()

    def __init__(self,caseDir):
	self.caseDir=caseDir
    #----------------------------------------------------------------------------------    
    def runIncompressibleSolver(self,mode):

	GEN=generalClass(self.caseDir);crFile=writeFileClassIncompressible(self.caseDir)

	crFile.UFileBasic();crFile.pFileBasic();crFile.kFileBasic();crFile.epsilonFileBasic();crFile.omegaFileBasic();crFile.nutFileBasic()

	ss=glob.glob(self.caseDir+'/Running')
	if len(ss)==0:
	    if mode=='initialize':
	        savedResult=GEN.getResult()
	        for ii in savedResult[0]:os.system('rm -r '+self.caseDir+'/'+ii)
	        if len(savedResult[1])!=0:os.system('rm -r '+self.caseDir+'/processor*')
	        os.system('rm -r '+self.caseDir+'/PyFoam*');os.system('rm -r '+self.caseDir+'/Gnuplotting.analyzed')
	    else:
	        inputForRun=self.incompressibleSet()
	        self.incompressibleRun(inputForRun[0],inputForRun[1],inputForRun[2],inputForRun[3],inputForRun[4],inputForRun[5],inputForRun[6],inputForRun[7])
	else:
	    md = gtk.MessageDialog(None,gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_ERROR,gtk.BUTTONS_CLOSE, "Solver is Running")
	    md.run();md.destroy()
    #----------------------------------------------------------------------------------
    def incompressibleSet(self):

	def getValue(inputfile,key):
	    f=open(inputfile,'r');ff=f.readlines();f.close()
	    itemlines=[]
	    for i in ff:itemlines.append(i[:-1])
	    items={}
	    for aa in itemlines:
	        aaa=aa.split(';')
		if len(aaa)==2:
		    keys=aaa[0].strip();values=aaa[1].strip()
	            items[keys]=values
		elif len(aaa)==1:
		    keys=aaa[0].strip();values='none'
	            items[keys]=values
		elif len(aaa)>2:
		    keys=aaa[0].strip()
		    aaa.remove(aaa[0])
		    values=''
		    for jj in aaa:values=values+jj
		    items[keys]=values
		else:
		    print 'Error!!! Something is wrong in the inputFile at ',aa
	    value=items[key]
	    print 'value of '+key+' is ',value
	    return value
	#--- Main program ---------------------------------------------------------------------------
	infile=self.caseDir+'/inputFile'
	meshFile=getValue(infile,'meshFile')

	startFrom=getValue(infile,'startFrom')
	startTime=getValue(infile,'startTime')
	stopAt=getValue(infile,'stopAt')
	endTime=getValue(infile,'endTime')
	deltaT=getValue(infile,'deltaT')
	adjustTimeStep=getValue(infile,'adjustTimeStep')
	maxCo=getValue(infile,'maxCo')
	maxDeltaT=getValue(infile,'maxDeltaT')
	writeControl=getValue(infile,'writeControl')
	writeInterval=getValue(infile,'writeInterval')
	purgeWrite=getValue(infile,'purgeWrite')
	writeFormat=getValue(infile,'writeFormat')
	writePrecision=getValue(infile,'writePrecision')
	writeCompression=getValue(infile,'writeCompression')
	nCore=getValue(infile,'nCore')
	smpOrCluster=getValue(infile,'smpOrCluster')
	hostfile=getValue(infile,'hostfile')
	#---------------------------------------
	ddtScheme=getValue(infile,'ddtScheme')
	gradScheme=getValue(infile,'gradScheme')
	laplacianScheme=getValue(infile,'laplacianScheme')
	interpolationScheme=getValue(infile,'interpolationScheme')
	snGrad=getValue(infile,'snGrad')
	divU=getValue(infile,'divU')
	divk=getValue(infile,'divk')
	pSolver=getValue(infile,'pSolver')
	USolver=getValue(infile,'USolver')
	correctphi=getValue(infile,'correctphi')
	nOuterCorrectors=getValue(infile,'nOuterCorrectors')
	nCorrectors=getValue(infile,'nCorrectors')
	nonOrthogonalCorrectors=getValue(infile,'nonOrthogonalCorrectors')
	residualControlp=getValue(infile,'residualControlp')
	residualControlU=getValue(infile,'residualControlU')
	residualControlk=getValue(infile,'residualControlk')
	relaxationp=getValue(infile,'relaxationp')
	relaxationU=getValue(infile,'relaxationU')
	relaxationk=getValue(infile,'relaxationk')
	potential=getValue(infile,'potentialInitialize')
	plotresidual=getValue(infile,'plotResidual')
	viscosity=getValue(infile,'viscosity')
	density=getValue(infile,'density')
	turbulenceModel=getValue(infile,'turbulenceModel')
	solver=getValue(infile,'solver')
	
	fvSolutionList=[pSolver,USolver,correctphi,nOuterCorrectors,nCorrectors,nonOrthogonalCorrectors,
			residualControlp,residualControlU,residualControlk,relaxationp,relaxationU,relaxationk]
	fvSchemesList=[ddtScheme,gradScheme,laplacianScheme,interpolationScheme,snGrad,divU,divk,solver]
	controlDictList=[startFrom,startTime,stopAt,endTime,deltaT,adjustTimeStep,maxCo,maxDeltaT,writeControl,
			 writeInterval,purgeWrite,writeFormat,writePrecision,writeCompression,solver]

	crFile=writeFileClassIncompressible(self.caseDir)

	aaa=glob.glob(self.caseDir+'/temporary/flowBoundaryConditions')
	if len(aaa)!=0:
	    f=open(aaa[0],'r');fff=f.readlines();f.close()
	    uxi=fff[0][:-1];uyi=fff[1][:-1];uzi=fff[2][:-1];pi=fff[3][:-1]
	    ff=fff[4:]
	    flowCondition={}
	    for i in range(len(ff)):
	        a1=ff[i][:-1].split(',')
	        flowCondition[a1[0]]=[a1[1],a1[2],a1[3],a1[4],a1[5]]
	else:
	    md=gtk.MessageDialog(None,gtk.DIALOG_MODAL,gtk.MESSAGE_INFO,gtk.BUTTONS_CLOSE,"flow B.C. is not set yet!!!")
	    md.run();md.destroy()

	aa=glob.glob(self.caseDir+'/temporary/turBoundaryConditions')
	if len(aa)!=0:
	    f=open(aa[0],'r');fff=f.readlines();f.close()
	    turCondition={}
	    turMethod=fff[0][:-1];ki=fff[1][:-1];ei=fff[2][:-1]
	    ff=fff[3:]
	    for i in range(len(ff)):
	        a1=ff[i][:-1].split(',')
	        turCondition[a1[0]]=[a1[1],a1[2],a1[3],a1[4]]
	else:
	    if turbulenceModel!='laminar':
		md=gtk.MessageDialog(None,gtk.DIALOG_MODAL,gtk.MESSAGE_INFO,gtk.BUTTONS_CLOSE,"turbulence B.C. is not set yet!!!")
		md.run();md.destroy()
	    else:
		ki='None';ei='None'

	internalField=[uxi,uyi,uzi,pi,ki,ei]

	pj=glob.glob(self.caseDir+'/temporary/porousJumpPatch_*')
	porousJumpPatch=[];DIL=[];jump={}
	if len(pj)!=0:
	    for i in range(len(pj)):
		f=open(pj[i],'r');ff=f.readlines();f.close()
		porousJumpPatch.append(ff[0][:-1])
		DIL.append(ff[1][:-1].split(' '))
		jump[porousJumpPatch[i]]=DIL[i]
		bb=porousJumpPatch[i][:-6]+'slave'
		jump[bb]=DIL[i]

	CHDICT=changeDictionary(self.caseDir)	
	CHDICT.changeInternalField(internalField,'incompressible')
	CHDICT.UFile(flowCondition,startFrom,startTime,nCore,density,internalField)
	CHDICT.pFile(flowCondition,jump,startFrom,startTime,nCore,internalField)
	if turbulenceModel!='laminar':
	    CHDICT.kFile(turCondition,startFrom,startTime,nCore,internalField)
	    CHDICT.nutFile(flowCondition,startFrom,startTime,nCore,turbulenceModel)
	    CHDICT.epsilonFile(turCondition,startFrom,startTime,nCore,internalField,turbulenceModel)
	    CHDICT.omegaFile(turCondition,startFrom,startTime,nCore,internalField,turbulenceModel)

	crFile.RASProperties(turbulenceModel)
	crFile.transportProperties(viscosity,density)
	crFile.turbulenceProperties(turbulenceModel)

	crFile.fvSolution(fvSolutionList,solver)
	crFile.fvSchemes(fvSchemesList)
	crFile.controlDict(controlDictList)
	crFile.decomposeParDict(nCore)

	po=glob.glob(self.caseDir+'/system/porousCellZone_*')
	mr=glob.glob(self.caseDir+'/system/MRFCellZone_*')
	fu=glob.glob(self.caseDir+'/system/fixedUCellZone_*')
	ms=glob.glob(self.caseDir+'/system/momSourceCellZone_*')
	crFile.fvOptions(po,mr,fu,ms)

	sm=glob.glob(self.caseDir+'/constant/SlidingMeshCellZone_*')
	if len(sm)!=0:crFile.dynamicMeshDict(sm)
	#-------------------------------------------------------------------------------------

	inputForRun=[solver,nCore,startFrom,plotresidual,smpOrCluster,hostfile,potential,startTime]
	return inputForRun

    #----------------------------------------------------------------------------------
    def incompressibleRun(self,solver,nCore,startFrom, plotresidual,smpOrCluster,hostfile,potential,startTime):

	GEN=generalClass(self.caseDir)

	results=GEN.getResult()
	serialResult=results[0];parallelResult=results[1]

	f=open(self.caseDir+'/runSolver','w')
	f.write('touch '+self.caseDir+'/Running \n')

	if potential=='yes':
	    f.write('cp -r '+self.caseDir+'/0 '+self.caseDir+'/0.org\n')
	    f.write('potentialFoam -writep -noFunctionObjects -case '+self.caseDir+'\n')

	aa=glob.glob(self.caseDir+'/postProcessing')
	if len(aa)!=0:f.write('rm -r '+self.caseDir+'/postProcessing \n')

	f.write('rm -r '+self.caseDir+'/PyFoam* \n')
	
	if nCore=='1':#serial
	    if startFrom=='startTime':
	        if startTime=='0':
		    GEN.clearResult(serialResult,'0');GEN.clearParallel()
	            if plotresidual=='yes':
		        f.write('pyFoamPlotRunner.py --no-continuity '+solver+' -case '+self.caseDir+' \n')
	            else:
		        f.write('pyFoamRunner.py '+solver+' -case '+self.caseDir+' \n')
		else:
		    serialResult.append(startTime)
		    if serialResult.count(startTime)==2:
		        serialResult.remove(startTime);serialResult.remove(startTime)
		        GEN.clearResult(serialResult,'0');GEN.clearParallel()
	                if plotresidual=='yes':
		            f.write('pyFoamPlotRunner.py --no-continuity '+solver+' -case '+self.caseDir+' \n')
	                else:
		            f.write('pyFoamRunner.py '+solver+' -case '+self.caseDir+' \n')
		    else:
			parallelResult.append(startTime)
			if parallelResult.count(startTime)==2:
			    f.write('reconstructPar -case +'/self.caseDir+' -time '+startTime+' \n')
			    f.write('rm -r '+self.caseDir+'/processor* \n')
	                    if plotresidual=='yes':
		                f.write('pyFoamPlotRunner.py --no-continuity '+solver+' -case '+self.caseDir+' \n')
	                    else:
		                f.write('pyFoamRunner.py '+solver+' -case '+self.caseDir+' \n')
			else:
			    md = gtk.MessageDialog(None,gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_ERROR,gtk.BUTTONS_CLOSE, "Error!!! There is not startTime folder")
			    md.run();md.destroy()
	    else:# seiral + latestTime
		if len(parallelResult)==0:GEN.clearParallel()
		else:
		    f.write('reconstructPar -case /'+self.caseDir+' -latestTime \n')
		    f.write('rm -r '+self.caseDir+'/processor* \n')
	        if plotresidual=='yes':
		    f.write('pyFoamPlotRunner.py --no-continuity '+solver+' -case '+self.caseDir+' \n')
	        else:
		    f.write('pyFoamRunner.py '+solver+' -case '+self.caseDir+' \n')

	else:#parallel
	    aa=glob.glob(self.caseDir+'/processor*');nCorePrev=str(len(aa))
	    if nCore==nCorePrev:
	        if startFrom=='startTime':
	            if startTime=='0':
		        GEN.clearResult(serialResult,'0');GEN.clearResult(parallelResult,'1')
	    	        if smpOrCluster=='SMP':
	        	    if plotresidual=='yes':
		    	        f.write('pyFoamPlotRunner.py --no-continuity mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	        	    else:
		    	        f.write('pyFoamRunner.py mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	    		else:
	        	    if plotresidual=='yes':
		    	        f.write('pyFoamPlotRunner.py --no-continuity mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	        	    else:
		    		f.write('pyFoamRunner.py  mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
		    else:
			parallelResult.append(startTime)
			if parallelResult.count(startTime)==2:
			    parallelResult.remove(startTime);parallelResult.remove(startTime);GEN.clearResult(parallelResult,'1')
			    GEN.clearResult(serialResult,'0')
	    		    if smpOrCluster=='SMP':
	        	        if plotresidual=='yes':
		    		    f.write('pyFoamPlotRunner.py --no-continuity mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	        		else:
		    		    f.write('pyFoamRunner.py mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	    		    else:
	        		if plotresidual=='yes':
		    		    f.write('pyFoamPlotRunner.py --no-continuity mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	        		else:
		    		    f.write('pyFoamRunner.py  mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
			else:
			    serialResult.append(startTime)
			    if serialResult.count(startTime)==2:
			        serialResult.remove(startTime);serialResult.remove(startTime);GEN.clearResult(serialResult,'0')
				GEN.clearParallel()
				f.write('decomposePar -force -case '+self.caseDir+' \n')
	    		        if smpOrCluster=='SMP':
	        		    if plotresidual=='yes':
		    		        f.write('pyFoamPlotRunner.py --no-continuity mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	        		    else:
		    		        f.write('pyFoamRunner.py mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	    		        else:
	        		    if plotresidual=='yes':
		    		        f.write('pyFoamPlotRunner.py --no-continuity mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	        		    else:
		    		        f.write('pyFoamRunner.py  mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
			    else:
			        md = gtk.MessageDialog(None,gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_ERROR,gtk.BUTTONS_CLOSE, "Error!!! There is not startTime folder")
			        md.run();md.destroy()
		else:# parallel + latestTime
		    GEN.clearResult(serialResult,'0')
	    	    if smpOrCluster=='SMP':
	                if plotresidual=='yes':
		            f.write('pyFoamPlotRunner.py --no-continuity mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	        	else:
		    	    f.write('pyFoamRunner.py mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	    	    else:
	        	if plotresidual=='yes':
		    	    f.write('pyFoamPlotRunner.py --no-continuity mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	        	else:
		    	    f.write('pyFoamRunner.py  mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')

	    else:#nCore!=nCorePrev
	        if startFrom=='startTime':
		    if startTime=='0':
		        GEN.clearResult(serialResult,'0')
			GEN.clearParallel()
			f.write('decomposePar -force -case '+self.caseDir+' \n')
	    	        if smpOrCluster=='SMP':
	        	    if plotresidual=='yes':
		    	        f.write('pyFoamPlotRunner.py --no-continuity mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	        	    else:
		    	        f.write('pyFoamRunner.py mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	    		else:
	        	    if plotresidual=='yes':
		    	        f.write('pyFoamPlotRunner.py --no-continuity mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	        	    else:
		    		f.write('pyFoamRunner.py  mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
		    else:
			parallelResult.append(startTime)
			if parallelResult.count(startTime)==2:
			    f.write('reconstructPar -case'+self.caseDir+' -time '+startTime+' \n')
			    f.write('rm -r '+self.caseDir+'/processor* \n')
			    f.write('decomposePar -force -case '+self.caseDir+' \n')
			    GEN.clearResult(serialResult,'0')
	    		    if smpOrCluster=='SMP':
	        	        if plotresidual=='yes':
		    		    f.write('pyFoamPlotRunner.py --no-continuity mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	        		else:
		    		    f.write('pyFoamRunner.py mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	    		    else:
	        		if plotresidual=='yes':
		    		    f.write('pyFoamPlotRunner.py --no-continuity mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	        		else:
		    		    f.write('pyFoamRunner.py  mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
			else:
			    serialResult.append(startTime)
			    if serialResult.count(startTime)==2:
			        serialResult.remove(startTime);serialResult.remove(startTime);GEN.clearResult(serialResult,'0')
				GEN.clearParallel()
				f.write('decomposePar -case '+self.caseDir+' \n')
	    		        if smpOrCluster=='SMP':
	        		    if plotresidual=='yes':
		    		        f.write('pyFoamPlotRunner.py --no-continuity mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	        		    else:
		    		        f.write('pyFoamRunner.py mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	    		        else:
	        		    if plotresidual=='yes':
		    		        f.write('pyFoamPlotRunner.py --no-continuity mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	        		    else:
		    		        f.write('pyFoamRunner.py  mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
			    else:
			        md = gtk.MessageDialog(None,gtk.DIALOG_DESTROY_WITH_PARENT,gtk.MESSAGE_ERROR,gtk.BUTTONS_CLOSE, "Error!!! There is not startTime folder")
			        md.run();md.destroy()
		else:# parallel + latestTime
		    if nCorePrev=='0':f.write('decomposePar -force -case '+self.caseDir+' \n')
		    else:
		        GEN.clearResult(serialResult,'0')
		        f.write('reconstructPar -latestTime -case '+self.caseDir+' \n')
		        f.write('decomposePar -force -case '+self.caseDir+' \n')
	    	    if smpOrCluster=='SMP':
	        	if plotresidual=='yes':
		    	    f.write('pyFoamPlotRunner.py --no-continuity mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	        	else:
		    	    f.write('pyFoamRunner.py mpirun -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	    	    else:
	        	if plotresidual=='yes':
		    	    f.write('pyFoamPlotRunner.py --no-continuity mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')
	        	else:
		    	    f.write('pyFoamRunner.py  mpirun --hostfile '+hostfile+' -np '+nCore+' '+solver+' -case '+self.caseDir+' -parallel \n')

	f.write('rm '+self.caseDir+'/Running \n')

	f.close()
	os.system('chmod +x '+self.caseDir+'/runSolver');os.system(self.caseDir+'/runSolver & ')






         

  


