#-*-coding:utf8-*-
#############################################################################
#    Baram : Graphic User Interface for OpenFOAM
#    Copyright held by original author
#############################################################################
# License                                                                       
#    This file is part of Baram.                                            
#
#    Baram is free software; you can redistribute it and/or modify it
#    under the terms of the GNU General Public License as published by the
#    Free Software Foundation; either version 3 of the License, or (at your
#    option) any later version.
#
#    Baram is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
#    for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Baram.  If not, see <http://www.gnu.org/licenses/>.
#
# Description
#    - Class for writing various files of heat transfer solver
#
# Author
#    NEXTFOAM Inc. All rights reserved.
##############################################################################

import math,os,glob

class writeFileClassHeatTransfer:
    def __init__(self,caseDir):
        self.caseDir=caseDir

    #-----------------------------------------------------------------------------
    def getBCName(self):
        aa=self.caseDir+'/constant/polyMesh/boundary'
        filea=open(aa);lines=filea.readlines();filea.close()
	for ii in lines:
	    if ii=='\n':lines.remove(ii)
	num=int(lines[16])
        line1=[];bcname=[]
        f=open(aa,'r')
        for i in range(15,len(lines)):
            a=(lines[i]).strip()
            if a=='{':line1.append(i)            
        for i in line1:
            b=(lines[i-1]).strip();bcname.append(b)
        return bcname
    #-----------------------------------------------------------------------------
    def getBCType(self):
        aa=self.caseDir+'/constant/polyMesh/boundary'
        filea=open(aa,'r');lines=filea.readlines();filea.close()
	for ii in lines:
	    if ii=='\n':lines.remove(ii)
	num=int(lines[16])
        line1=[];bctype=[]
        f=open(aa,'r')
        for i in range(15,len(lines)):
            a=(lines[i]).strip()
            if a=='{':line1.append(i)            
        for i in line1:
            bb=(lines[i+1]).split();bb1=(bb[1])[:-1];bctype.append(bb1)
        return bctype
    #-----------------------------------------------------------------------------
    def inputfile(self,inputs):

	tt=[]
	tt.append('caseDir                        ; ');tt.append('meshFile                       ; ')
        tt.append('startFrom                      ; ');tt.append('startTime                      ; ')
        tt.append('stopAt                         ; ');tt.append('endTime                        ; ')
        tt.append('deltaT                         ; ');tt.append('adjustTimeStep                 ; ')
        tt.append('maxCo                          ; ');tt.append('maxDeltaT                      ; ')
        tt.append('writeControl                   ; ');tt.append('writeInterval                  ; ')
	tt.append('purgeWrite                     ; ');tt.append('writeFormat                    ; ')
	tt.append('writePrecision                 ; ');tt.append('writeCompression               ; ')
	tt.append('nCore                          ; ');tt.append('smpOrCluster                   ; ')
	tt.append('hostfile                       ; ');tt.append('ddtScheme                      ; ')
        tt.append('gradScheme                     ; ');tt.append('laplacianScheme                ; ')
        tt.append('interpolationScheme            ; ');tt.append('snGrad                         ; ')
        tt.append('divU                           ; ');tt.append('divk                           ; ')
        tt.append('divh                           ; ');tt.append('pSolver                        ; ')
        tt.append('USolver                        ; ');tt.append('hSolver                        ; ')
        tt.append('momentumPredictor              ; ');tt.append('nOuterCorrectors               ; ')
        tt.append('nCorrectors                    ; ');tt.append('nonOrthogonalCorrectors        ; ')
        tt.append('relaxationp                    ; ');tt.append('relaxationU                    ; ')
        tt.append('relaxationk                    ; ');tt.append('relaxationh                    ; ')
        tt.append('plotResidual                   ; ');tt.append('gravity                        ; ')
	tt.append('turbulenceModel                ; ');tt.append('operatingPressure              ; ')
	tt.append('solver                         ; ');tt.append('potentialInitialize            ; ')

        f=open(self.caseDir+'/inputFile','w')
	for i in range(len(inputs)):f.write(tt[i]+inputs[i]+'\n')
	f.close()
    #-------------------------------------------------------------------------------------
    def foamComment(self):
	fc=[]
        fc.append('/*--------------------------------*- C++ -*----------------------------------*\ \n')
        fc.append('| =========                 |                                                 |\n')
        fc.append('| \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox           |\n')
        fc.append('|  \\    /   O peration     | Version:  2.3.0                                 |\n')
        fc.append('|   \\  /    A nd           | Web:      www.OpenFOAM.org                      |\n')
        fc.append('|    \\/     M anipulation  |                                                 |\n')
        fc.append('\*---------------------------------------------------------------------------*/\n')	
	return fc
    #------------------------------------------------------------------------------
    def header(self,location,obj):
	he=[]
        he.append('FoamFile\n')
        he.append('{\n')
        he.append('    version     2.0;\n')
        he.append('    format      ascii;\n')
        he.append('    class       dictionary;\n')
        he.append('    location    "'+location+'";\n')
        he.append('    object      '+obj+';\n')
        he.append('}\n')
	he.append('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n')
	return he
    #------------------------------------------------------------------------------
    def BCheader(self,classtype,obj):
	he=[]
        he.append('FoamFile\n')
        he.append('{\n')
        he.append('    version     2.0;\n')
        he.append('    format      ascii;\n')
        he.append('    class       '+classtype+';\n')
        he.append('    object      '+obj+';\n')
        he.append('}\n')
	he.append('// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n')
	return he
    #------------------------------------------------------------------------------
    def regionProperties(self,names,fs):
	fcom=self.foamComment()
	head=self.BCheader('dictionary','regionProperties')
	hhh=fcom+head
        f=open(self.caseDir+'/constant/regionProperties','w')
	for aaa in hhh:f.write(aaa)	
        f.write('regions\n')
        f.write('(\n')
        f.write('    fluid    (')
	for i in range(len(names)):
	    if fs[i]=='Fluid':
	        f.write(names[i]+' ')	
	f.write(')\n')
        f.write('    solid    (')
	for i in range(len(names)):
	    if fs[i]=='Solid':
	        f.write(names[i]+' ')	
	f.write(')\n')
        f.write(');\n')
	f.close()
    #------------------------------------------------------------------------------
    def UFile(self,flowCondition,internalField):
	#internalField=[uxi,uyi,uzi,pi,ki,ei]
	uini='('+internalField[0]+' '+internalField[1]+' '+internalField[2]+')'

	fcom=self.foamComment()
	head=self.BCheader('volVectorField','U')
	hhh=fcom+head

	for i in range(1):
	    bcname=self.getBCName();bctype=self.getBCType()

            f=open(self.caseDir+'/0/U','w')
	    for aaa in hhh:f.write(aaa)	
            f.write('dimensions      [0 1 -1 0 0 0 0];\n')
            f.write('internalField   uniform '+uini+';\n')
            f.write('boundaryField\n')
            f.write('{\n')
            f.write('    #include "${WM_PROJECT_DIR}/etc/caseDicts/setConstraintTypes"\n')

	    for j in range(len(bcname)):
		if bctype[j]=='wall' or bctype[j]=='patch' or bctype[j]=='mappedWall':
		    condition=flowCondition[bcname[j]]
		    patchType=condition[1]
		    patchValue=[condition[2],condition[3],condition[4]]

		    f.write('    '+bcname[j]+'\n')
                    f.write('    {\n')
		    if patchType=='wall':
            	        f.write('        type    fixedValue;\n')
            	        f.write('        value   uniform (0 0 0);\n')
		    elif patchType=='moving-wall':
            	        f.write('        type    fixedValue;\n')
            	        f.write('        value   uniform ('+patchValue[0]+' '+patchValue[1]+' '+patchValue[2]+');\n')
		    elif patchType=='fixed-U':
            	        f.write('        type    fixedValue;\n')
            	        f.write('        value   uniform ('+patchValue[0]+' '+patchValue[1]+' '+patchValue[2]+');\n')
		    elif patchType=='surfaceNormal-U':
            	        f.write('        type    surfaceNormalFixedValue;\n')
            	        f.write('        refValue   uniform -'+patchValue[0]+';\n')
		    elif patchType=='volume-flowrate':
            	        f.write('        type    flowRateInletVelocity;\n')
            	        f.write('        volumetricFlowRate    '+patchValue[0]+';\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='mass-flowrate':
            	        f.write('        type    flowRateInletVelocity;\n')
            	        f.write('        massFlowRate    '+patchValue[0]+';\n')
            	        f.write('        rho         rho;\n')
            	        f.write('        rhoInlet    '+rho+';\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='total-pressure':
            	        f.write('        type    zeroGradient;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='outlet':
            	        f.write('        type    pressureInletOutletVelocity;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='movingWallVelocity':
            	        f.write('        type    movingWallVelocity;\n')
            	        f.write('        value   uniform (0 0 0);\n')
		    elif patchType=='rotatingWallVelocity':
			if patchValue[1]=='x':axis='(1 0 0)'
			elif patchValue[1]=='y':axis='(0 1 0)'
			elif patchValue[1]=='z':axis='(0 0 1)'
			elif patchValue[1]=='-x':axis='(-1 0 0)'
			elif patchValue[1]=='-y':axis='(0 -1 0)'
			elif patchValue[1]=='-z':axis='(0 0 -1)'
			omega=float(patchValue[2])*2*3.141592/60;omega=str(omega)
            	        f.write('        type    rotatingWallVelocity;\n')
            	        f.write('        origin  '+patchValue[0]+';\n')
            	        f.write('        axis    '+axis+';\n')
            	        f.write('        omega   '+omega+';\n')
            	        f.write('        value   uniform (0 0 0);\n')
            	    f.write('    }\n')
            f.write('}\n')
	    f.close()
    #------------------------------------------------------------------------------
    def pFile(self,flowCondition,internalField,operatingPressure,jump):
	#internalField=[uxi,uyi,uzi,pi,ki,ei]

	fcom=self.foamComment()
	head=self.BCheader('volScalarField','p')
	hhh=fcom+head

	if operatingPressure=='none':
	    pini=str(float(internalField[3])+101325)
	    operatingPressure='101325'
	else:
	    pini='101325'

	for i in range(1):

	    bcname=self.getBCName();bctype=self.getBCType()

            f=open(self.caseDir+'/0/p','w')
	    for aaa in hhh:f.write(aaa)	
            f.write('dimensions      [1 -1 -2 0 0 0 0];\n')
            f.write('internalField   uniform '+pini+';\n')
            f.write('boundaryField\n')
            f.write('{\n')
            f.write('    #include "${WM_PROJECT_DIR}/etc/caseDicts/setConstraintTypes"\n')

	    for j in range(len(bcname)):

		if bctype[j]=='wall' or bctype[j]=='patch':
		    condition=flowCondition[bcname[j]]
		    patchType=condition[1]
		    patchValue=[condition[2],condition[3],condition[4]]

		    f.write('    '+bcname[j]+'\n')
                    f.write('    {\n')
		    if patchType=='wall':
            	        f.write('        type    calculated;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='moving-wall':
            	        f.write('        type    calculated;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='movingWallVelocity':
            	        f.write('        type    calculated;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='rotatingWallVelocity':
            	        f.write('        type    calculated;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='fixed-U':
            	        f.write('        type    calculated;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='surfaceNormal-U':
            	        f.write('        type    calculated;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='volume-flowrate':
            	        f.write('        type    calculated;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='mass-flowrate':
            	        f.write('        type    calculated;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='total-pressure':
            	        f.write('        type    calculated;\n')
            	        f.write('        gamma   1.4;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='outlet':
            	        f.write('        type    calculated;\n')
            	        f.write('        value   $internalField;\n')
            	    f.write('    }\n')
		elif bctype[j]=='mappedWall':
		    f.write('    '+bcname[j]+'\n')
                    f.write('    {\n')
            	    f.write('        type    calculated;\n')
            	    f.write('        value   $internalField;\n')
            	    f.write('    }\n')
		elif bctype[j]=='cyclic':
		    aa=jump.keys();bb=aa.count(bcname[j])
		    if bb!=0:
		        f.write('    '+bcname[j]+'\n')
                        f.write('    {\n')
            	        f.write('        type    calculated;\n')
            	        f.write('        value   $internalField;\n')
            	        f.write('    }\n')

            f.write('}\n')
	    f.close()
    #------------------------------------------------------------------------------
    def p_rghFile(self,flowCondition,internalField,jump,operatingPressure):
	#internalField=[uxi,uyi,uzi,pi,ki,ei]
	fcom=self.foamComment()
	head=self.BCheader('volScalarField','p_rgh')
	hhh=fcom+head

	if operatingPressure=='none':
	    #pini=str(float(internalField[3])+101325)
	    pini=internalField[3]
	    operatingPressure='101325'
	else:
	    pini='101325'

	for i in range(1):

	    bcname=self.getBCName();bctype=self.getBCType()

            f=open(self.caseDir+'/0/p_rgh','w')
	    for aaa in hhh:f.write(aaa)	
            f.write('dimensions      [1 -1 -2 0 0 0 0];\n')
            f.write('internalField   uniform '+pini+';\n')
            f.write('boundaryField\n')
            f.write('{\n')
            f.write('    #include "${WM_PROJECT_DIR}/etc/caseDicts/setConstraintTypes"\n')

	    for j in range(len(bcname)):

		print 'p BC for patch : ',j,bcname[j],bctype[j]

		if bctype[j]=='wall' or bctype[j]=='patch':
		    condition=flowCondition[bcname[j]]
		    patchType=condition[1]
		    patchValue=[condition[2],condition[3],condition[4]]
		    if patchValue[0]!='None':
		        pvalue=str(101325-float(operatingPressure)+float(patchValue[0])) 		   

		    f.write('    '+bcname[j]+'\n')
                    f.write('    {\n')
		    if patchType=='wall':
            	        f.write('        type    fixedFluxPressure;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='moving-wall':
            	        f.write('        type    fixedFluxPressure;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='movingWallVelocity':
            	        f.write('        type    fixedFluxPressure;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='rotatingWallVelocity':
            	        f.write('        type    fixedFluxPressure;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='fixed-U':
            	        f.write('        type    fixedFluxPressure;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='surfaceNormal-U':
            	        f.write('        type    fixedFluxPressure;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='volume-flowrate':
            	        f.write('        type    fixedFluxPressure;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='mass-flowrate':
            	        f.write('        type    fixedFluxPressure;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='total-pressure':
            	        f.write('        type    totalPressure;\n')
            	        f.write('        gamma   1.4;\n')
            	        #f.write('        p0      uniform '+patchValue[0]+';\n')
            	        f.write('        p0      uniform '+pvalue+';\n')
		    elif patchType=='outlet':
            	        f.write('        type    fixedValue;\n')
            	        #f.write('        value   uniform '+patchValue[0]+';\n')
            	        f.write('        value   uniform '+pvalue+';\n')
            	    f.write('    }\n')
		elif bctype[j]=='mappedWall':
		    f.write('    '+bcname[j]+'\n')
                    f.write('    {\n')
            	    f.write('        type    fixedFluxPressure;\n')
            	    f.write('        value   $internalField;\n')
            	    f.write('    }\n')
		elif bctype[j]=='cyclic':
		    aa=jump.keys();bb=aa.count(bcname[j])
		    if bb!=0:
		        f.write('    '+bcname[j]+'\n')
                        f.write('    {\n')
            	        f.write('        type    porousBafflePressure;\n')
            	        f.write('        patchType    cyclic;\n')
            	        f.write('        value   uniform '+pini+';\n')
            	        f.write('        D       '+jump[bcname[j]][0]+';\n')
            	        f.write('        I       '+jump[bcname[j]][1]+';\n')
            	        f.write('        length  '+jump[bcname[j]][2]+';\n')
            	        f.write('    }\n')

            f.write('}\n')
	    f.close()
    #------------------------------------------------------------------------------
    def TFile(self,thermalCondition,internalField):

	fcom=self.foamComment()
	head=self.BCheader('volScalarField','T')
	hhh=fcom+head

	tini=internalField[6]

	for i in range(1):

	    bcname=self.getBCName();bctype=self.getBCType() 

            f=open(self.caseDir+'/0/T','w')
	    for aaa in hhh:f.write(aaa)	
            f.write('dimensions      [0 0 0 1 0 0 0];\n')
            f.write('internalField   uniform '+tini+';\n')
            f.write('boundaryField\n')
            f.write('{\n')
            f.write('    #include "${WM_PROJECT_DIR}/etc/caseDicts/setConstraintTypes"\n')

	    for j in range(len(bcname)):
		if bctype[j]=='wall' or bctype[j]=='patch':
		    condition=thermalCondition[bcname[j]]
		    patchType=condition[0]
		    patchValue=[condition[1],condition[2],condition[3]]

		    f.write('    '+bcname[j]+'\n')
                    f.write('    {\n')
		    if patchType=='zeroGradient':
            	        f.write('        type    zeroGradient;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='fixedValue':
            	        f.write('        type    fixedValue;\n')
            	        f.write('        value   uniform '+patchValue[0]+';\n')
		    elif patchType=='heatflux':
            	        f.write('        type    externalWallHeatFluxTemperature;\n')
            	        f.write('        kappa   fluidThermo;\n')
            	        f.write('        q       uniform '+patchValue[0]+';\n')
            	        f.write('        kappaName   none;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='convection':
            	        f.write('        type    externalWallHeatFluxTemperature;\n')
            	        f.write('        kappa   fluidThermo;\n')
            	        f.write('        Ta      uniform '+patchValue[0]+';\n')
            	        f.write('        h       uniform '+patchValue[1]+';\n')
            	        f.write('        kappaName   none;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='coupled':
            	        f.write('        type            compressible::turbulentTemperatureRadCoupledMixed;\n')
            	        f.write('        value           uniform 300;\n')
            	        f.write('        Tnbr            T;\n')
            	        f.write('        kappa           fluidThermo;\n')
            	        f.write('        kappaName       none;\n')
            	    f.write('    }\n')
		elif bctype[j]=='mappedWall':
		    f.write('    '+bcname[j]+'\n')
                    f.write('    {\n')
            	    f.write('        type    compressible::turbulentTemperatureRadCoupledMixed;\n')
            	    f.write('        value   uniform 300;\n')
            	    f.write('        Tnbr    T;\n')
            	    f.write('        kappa   fluidThermo;\n')
            	    f.write('        kappaName   none;\n')
            	    f.write('    }\n')
            f.write('}\n')
	    f.close()
    #------------------------------------------------------------------------------
    def kFile(self,flowCondition,internalField,turCondition,turMethod,rho,mu):
	uref='1'#####

	if rho=='perfectGas':rho=1.2
	if mu[:10]=='Sutherland':mu='1e-5'

	keys=flowCondition.keys();fix=[];sn=[];fr=[];tp=[]
	for i in range(len(keys)):
	    if flowCondition[keys[i]][1]=='fixed-U':fix.append(keys[i])
	    elif flowCondition[keys[i]][1]=='surfaceNormal-U':sn.append(keys[i])
	    elif flowCondition[keys[i]][1]=='volume-flowrate' or flowCondition[keys[i]][1]=='mass-flowrate':fr.append(keys[i])
	    elif flowCondition[keys[i]][1]=='total-pressure':tp.append(keys[i])

	umag={};kvalue={};evalue={};ovalue={}
	for i in range(len(fix)):
	    ux=flowCondition[fix[i]][2];uy=flowCondition[fix[i]][3];uz=flowCondition[fix[i]][4]
	    uu=(float(ux)**2+float(uy)**2+float(uz)**2)**0.5;umag[fix[i]]=str(uu)
	    kvalue[fix[i]]=1.5*(float(umag[fix[i]])*float(turCondition[fix[i]][2])*0.01)**2
	    evalue[fix[i]]=float(rho)*0.09*kvalue[fix[i]]**2/(float(mu)*float(turCondition[fix[i]][3]))
	    ovalue[fix[i]]=float(rho)*kvalue[fix[i]]/(float(mu)*float(turCondition[fix[i]][3]))
	for i in range(len(sn)):
	    umag[sn[i]]=flowCondition[sn[i]][2]
	    kvalue[sn[i]]=1.5*(float(umag[sn[i]])*float(turCondition[sn[i]][2])*0.01)**2
	    evalue[sn[i]]=float(rho)*0.09*kvalue[sn[i]]**2/(float(mu)*float(turCondition[sn[i]][3]))
	    ovalue[sn[i]]=float(rho)*kvalue[sn[i]]/(float(mu)*float(turCondition[sn[i]][3]))
	for i in range(len(fr)):
	    umag[fr[i]]=uref
	    kvalue[fr[i]]=1.5*(float(umag[fr[i]])*float(turCondition[fr[i]][2])*0.01)**2	
	    evalue[fr[i]]=float(rho)*0.09*kvalue[fr[i]]**2/(float(mu)*float(turCondition[fr[i]][3]))
	    ovalue[fr[i]]=float(rho)*kvalue[fr[i]]/(float(mu)*float(turCondition[fr[i]][3]))
	for i in range(len(tp)):
	    umag[tp[i]]=uref
	    kvalue[tp[i]]=1.5*(float(umag[tp[i]])*float(turCondition[tp[i]][2])*0.01)**2	
	    evalue[tp[i]]=float(rho)*0.09*kvalue[tp[i]]**2/(float(mu)*float(turCondition[tp[i]][3]))
	    ovalue[tp[i]]=float(rho)*kvalue[tp[i]]/(float(mu)*float(turCondition[tp[i]][3]))

	ki=internalField[4];ei=internalField[5]
	if turMethod=='intensity/vis.ratio':
	    kini=1.5*(float(uref)*float(ki)*0.01)**2
	    eini=float(rho)*0.09*kini**2/(float(mu)*float(ei))
	    oini=float(rho)*kini/(float(mu)*float(ei))
	    kini=str(kini);eini=str(eini);oini=str(oini)
	else:
	    kini=ki;eini=ei;oini=ei

	fcom=self.foamComment()
	head=self.BCheader('volScalarField','k')
	hhh=fcom+head

	for i in range(1):

	    bcName=self.getBCName();bctype=self.getBCType()

            f=open(self.caseDir+'/0/k','w')
	    for aaa in hhh:f.write(aaa)	
            f.write('dimensions      [0 2 -2 0 0 0 0];\n')
            f.write('internalField   uniform '+kini+';\n')
            f.write('boundaryField\n')
            f.write('{\n')
            f.write('    #include "${WM_PROJECT_DIR}/etc/caseDicts/setConstraintTypes"\n')

	    for j in range(len(bcName)):
		if bctype[j]=='wall' or bctype[j]=='patch':
		    condition=turCondition[bcName[j]]
		    patchType=condition[1]
		    #patchValue=[condition[2],condition[3],condition[4]]

		    f.write('    '+bcName[j]+'\n')
                    f.write('    {\n')
		    if patchType=='wallFunction':
            	        f.write('        type    compressible::kqRWallFunction;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='fixed-Value':
            	        f.write('        type    fixedValue;\n')
			if turMethod=='intensity/vis.ratio':f.write('        value   uniform '+str(kvalue[bcName[j]])+';\n')
			else:f.write('        value   uniform '+turCondition[bcName[j]][2]+';\n')
		    elif patchType=='zeroGradient':
            	        f.write('        type    zeroGradient;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='inletOutlet':
            	        f.write('        type    inletOutlet;\n')
            	        f.write('        inletValue   uniform '+turCondition[bcName[j]][2]+';\n')
            	        f.write('        value   $internalField;\n')
            	    f.write('    }\n')
		elif bctype[j]=='mappedWall':
		    f.write('    '+bcName[j]+'\n')
                    f.write('    {\n')
            	    f.write('        type    compressible::kqRWallFunction;\n')
            	    f.write('        value   $internalField;\n')
            	    f.write('    }\n')
            f.write('}\n')
	    f.close()
    #------------------------------------------------------------------------------
    def epsilonFile(self,flowCondition,internalField,turCondition,turMethod,rho,mu):
	uref='1'#####
	if rho=='perfectGas':rho=1.2
	if mu[:10]=='Sutherland':mu='1e-5'

	keys=flowCondition.keys();fix=[];sn=[];fr=[];tp=[]
	for i in range(len(keys)):
	    if flowCondition[keys[i]][1]=='fixed-U':fix.append(keys[i])
	    elif flowCondition[keys[i]][1]=='surfaceNormal-U':sn.append(keys[i])
	    elif flowCondition[keys[i]][1]=='volume-flowrate' or turCondition[keys[i]][1]=='mass-flowrate':fr.append(keys[i])
	    elif flowCondition[keys[i]][1]=='total-pressure':tp.append(keys[i])

	umag={};kvalue={};evalue={};ovalue={}
	for i in range(len(fix)):
	    ux=flowCondition[fix[i]][2];uy=flowCondition[fix[i]][3];uz=flowCondition[fix[i]][4]
	    uu=(float(ux)**2+float(uy)**2+float(uz)**2)**0.5;umag[fix[i]]=str(uu)
	    kvalue[fix[i]]=1.5*(float(umag[fix[i]])*float(turCondition[fix[i]][2])*0.01)**2
	    evalue[fix[i]]=float(rho)*0.09*kvalue[fix[i]]**2/(float(mu)*float(turCondition[fix[i]][3]))
	    ovalue[fix[i]]=float(rho)*kvalue[fix[i]]/(float(mu)*float(turCondition[fix[i]][3]))
	for i in range(len(sn)):
	    umag[sn[i]]=flowCondition[sn[i]][2]
	    kvalue[sn[i]]=1.5*(float(umag[sn[i]])*float(turCondition[sn[i]][2])*0.01)**2
	    evalue[sn[i]]=float(rho)*0.09*kvalue[sn[i]]**2/(float(mu)*float(turCondition[sn[i]][3]))
	    ovalue[sn[i]]=float(rho)*kvalue[sn[i]]/(float(mu)*float(turCondition[sn[i]][3]))
	for i in range(len(fr)):
	    umag[fr[i]]=uref
	    kvalue[fr[i]]=1.5*(float(umag[fr[i]])*float(turCondition[fr[i]][2])*0.01)**2	
	    evalue[fr[i]]=float(rho)*0.09*kvalue[fr[i]]**2/(float(mu)*float(turCondition[fr[i]][3]))
	    ovalue[fr[i]]=float(rho)*kvalue[fr[i]]/(float(mu)*float(turCondition[fr[i]][3]))
	for i in range(len(tp)):
	    umag[tp[i]]=uref
	    kvalue[tp[i]]=1.5*(float(umag[tp[i]])*float(turCondition[tp[i]][2])*0.01)**2	
	    evalue[tp[i]]=float(rho)*0.09*kvalue[tp[i]]**2/(float(mu)*float(turCondition[tp[i]][3]))
	    ovalue[tp[i]]=float(rho)*kvalue[tp[i]]/(float(mu)*float(turCondition[tp[i]][3]))

	ki=internalField[4];ei=internalField[5]
	if turMethod=='intensity/vis.ratio':
	    kini=1.5*(float(uref)*float(ki)*0.01)**2
	    eini=float(rho)*0.09*kini**2/(float(mu)*float(ei))
	    oini=float(rho)*kini/(float(mu)*float(ei))
	    kini=str(kini);eini=str(eini);oini=str(oini)
	else:
	    kini=ki;eini=ei;oini=ei

	fcom=self.foamComment()
	head=self.BCheader('volScalarField','epsilon')
	hhh=fcom+head

	for i in range(1):
	    bcName=self.getBCName();bctype=self.getBCType()

            f=open(self.caseDir+'/0/epsilon','w')
	    for aaa in hhh:f.write(aaa)	
            f.write('dimensions      [0 2 -3 0 0 0 0];\n')
            f.write('internalField   uniform '+eini+';\n')
            f.write('boundaryField\n')
            f.write('{\n')
            f.write('    #include "${WM_PROJECT_DIR}/etc/caseDicts/setConstraintTypes"\n')

	    for j in range(len(bcName)):
		if bctype[j]=='wall' or bctype[j]=='patch':
		    condition=turCondition[bcName[j]]
		    patchType=condition[1]
		    #patchValue=[condition[2],condition[3],condition[4]]

		    f.write('    '+bcName[j]+'\n')
                    f.write('    {\n')
		    if patchType=='wallFunction':
            	        f.write('        type    compressible::epsilonWallFunction;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='fixed-Value':
            	        f.write('        type    fixedValue;\n')
			if turMethod=='intensity/vis.ratio':f.write('        value   uniform '+str(evalue[bcName[j]])+';\n')
			else:f.write('        value   uniform '+turCondition[bcName[j]][3]+';\n')
		    elif patchType=='zeroGradient':
            	        f.write('        type    zeroGradient;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='inletOutlet':
            	        f.write('        type    inletOutlet;\n')
            	        f.write('        inletValue   uniform '+turCondition[bcName[j]][3]+';\n')
            	        f.write('        value   $internalField;\n')
            	    f.write('    }\n')
		elif bctype[j]=='mappedWall':
		    f.write('    '+bcName[j]+'\n')
                    f.write('    {\n')
            	    f.write('        type    compressible::epsilonWallFunction;\n')
            	    f.write('        value   $internalField;\n')
            	    f.write('    }\n')
            f.write('}\n')
	    f.close()
    #------------------------------------------------------------------------------
    def omegaFile(self,flowCondition,internalField,turCondition,turMethod,rho,mu):
	uref='1'#####
	if rho=='perfectGas':rho=1.2
	if mu[:10]=='Sutherland':mu='1e-5'

	keys=flowCondition.keys();fix=[];sn=[];fr=[];tp=[]
	for i in range(len(keys)):
	    if flowCondition[keys[i]][1]=='fixed-U':fix.append(keys[i])
	    elif flowCondition[keys[i]][1]=='surfaceNormal-U':sn.append(keys[i])
	    elif flowCondition[keys[i]][1]=='volume-flowrate' or turCondition[keys[i]][1]=='mass-flowrate':fr.append(keys[i])
	    elif flowCondition[keys[i]][1]=='total-pressure':tp.append(keys[i])

	umag={};kvalue={};evalue={};ovalue={}
	for i in range(len(fix)):
	    ux=flowCondition[fix[i]][2];uy=flowCondition[fix[i]][3];uz=flowCondition[fix[i]][4]
	    uu=(float(ux)**2+float(uy)**2+float(uz)**2)**0.5;umag[fix[i]]=str(uu)
	    kvalue[fix[i]]=1.5*(float(umag[fix[i]])*float(turCondition[fix[i]][2])*0.01)**2
	    evalue[fix[i]]=float(rho)*0.09*kvalue[fix[i]]**2/(float(mu)*float(turCondition[fix[i]][3]))
	    ovalue[fix[i]]=float(rho)*kvalue[fix[i]]/(float(mu)*float(turCondition[fix[i]][3]))
	for i in range(len(sn)):
	    umag[sn[i]]=flowCondition[sn[i]][2]
	    kvalue[sn[i]]=1.5*(float(umag[sn[i]])*float(turCondition[sn[i]][2])*0.01)**2
	    evalue[sn[i]]=float(rho)*0.09*kvalue[sn[i]]**2/(float(mu)*float(turCondition[sn[i]][3]))
	    ovalue[sn[i]]=float(rho)*kvalue[sn[i]]/(float(mu)*float(turCondition[sn[i]][3]))
	for i in range(len(fr)):
	    umag[fr[i]]=uref
	    kvalue[fr[i]]=1.5*(float(umag[fr[i]])*float(turCondition[fr[i]][2])*0.01)**2	
	    evalue[fr[i]]=float(rho)*0.09*kvalue[fr[i]]**2/(float(mu)*float(turCondition[fr[i]][3]))
	    ovalue[fr[i]]=float(rho)*kvalue[fr[i]]/(float(mu)*float(turCondition[fr[i]][3]))
	for i in range(len(tp)):
	    umag[tp[i]]=uref
	    kvalue[tp[i]]=1.5*(float(umag[tp[i]])*float(turCondition[tp[i]][2])*0.01)**2	
	    evalue[tp[i]]=float(rho)*0.09*kvalue[tp[i]]**2/(float(mu)*float(turCondition[tp[i]][3]))
	    ovalue[tp[i]]=float(rho)*kvalue[tp[i]]/(float(mu)*float(turCondition[tp[i]][3]))

	ki=internalField[4];ei=internalField[5]
	if turMethod=='intensity/vis.ratio':
	    kini=1.5*(float(uref)*float(ki)*0.01)**2
	    eini=float(rho)*0.09*kini**2/(float(mu)*float(ei))
	    oini=float(rho)*kini/(float(mu)*float(ei))
	    kini=str(kini);eini=str(eini);oini=str(oini)
	else:
	    kini=ki;eini=ei;oini=ei

	fcom=self.foamComment()
	head=self.BCheader('volScalarField','omega')
	hhh=fcom+head

	for i in range(1):
	    bcName=self.getBCName();bctype=self.getBCType()

            f=open(self.caseDir+'/0/omega','w')
	    for aaa in hhh:f.write(aaa)	
            f.write('dimensions      [0 0 -1 0 0 0 0];\n')
            f.write('internalField   uniform '+oini+';\n')
            f.write('boundaryField\n')
            f.write('{\n')
            f.write('    #include "${WM_PROJECT_DIR}/etc/caseDicts/setConstraintTypes"\n')

	    for j in range(len(bcName)):
		if bctype[j]=='wall' or bctype[j]=='patch':
		    condition=turCondition[bcName[j]]
		    patchType=condition[1]
		    #patchValue=[condition[2],condition[3],condition[4]]

		    f.write('    '+bcName[j]+'\n')
                    f.write('    {\n')
		    if patchType=='wallFunction':
            	        f.write('        type    compressible::omegaWallFunction;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='fixed-Value':
            	        f.write('        type    fixedValue;\n')
			if turMethod=='intensity/vis.ratio':f.write('        value   uniform '+str(ovalue[bcName[j]])+';\n')
			else:f.write('        value   uniform '+turCondition[bcName[j]][3]+';\n')
		    elif patchType=='zeroGradient':
            	        f.write('        type    zeroGradient;\n')
            	        f.write('        value   $internalField;\n')
		    elif patchType=='inletOutlet':
            	        f.write('        type    inletOutlet;\n')
            	        f.write('        inletValue   uniform '+turCondition[bcName[j]][3]+';\n')
            	        f.write('        value   $internalField;\n')
            	    f.write('    }\n')
		elif bctype[j]=='mappedWall':
		    f.write('    '+bcName[j]+'\n')
                    f.write('    {\n')
            	    f.write('        type    compressible::omegaWallFunction;\n')
            	    f.write('        value   $internalField;\n')
            	    f.write('    }\n')
            f.write('}\n')
	    f.close()
    #------------------------------------------------------------------------------
    def alphatFile(self):
	fcom=self.foamComment()
	head=self.BCheader('volScalarField','alphat')
	hhh=fcom+head

	for i in range(1):
	    bcName=self.getBCName();bctype=self.getBCType()

            f=open(self.caseDir+'/0/alphat','w')
	    for aaa in hhh:f.write(aaa)	
            f.write('dimensions      [1 -1 -1 0 0 0 0];\n')
            f.write('internalField   uniform 0;\n')
            f.write('boundaryField\n')
            f.write('{\n')
            f.write('    #include "${WM_PROJECT_DIR}/etc/caseDicts/setConstraintTypes"\n')

	    for j in range(len(bcName)):
		if bctype[j]=='wall':
		    f.write('    '+bcName[j]+'\n')
                    f.write('    {\n')
            	    f.write('        type    compressible::alphatJayatillekeWallFunction;\n')
            	    f.write('        prt     0.85;\n')
            	    f.write('        value   $internalField;\n')
            	    f.write('    }\n')
		elif bctype[j]=='patch':
		    f.write('    '+bcName[j]+'\n')
                    f.write('    {\n')
            	    f.write('        type    calculated;\n')
            	    f.write('        value   $internalField;\n')
            	    f.write('    }\n')
            f.write('}\n')
	    f.close()
    #------------------------------------------------------------------------------
    def GFile(self,emissivities):
	fcom=self.foamComment()
	head=self.BCheader('volScalarField','G')
	hhh=fcom+head

	for i in range(1):
	    bcName=self.getBCName();bctype=self.getBCType()

            f=open(self.caseDir+'/0/G','w')
	    for aaa in hhh:f.write(aaa)	
            f.write('dimensions      [1 0 -3 0 0 0 0];\n')
            f.write('internalField   uniform 0;\n')
            f.write('boundaryField\n')
            f.write('{\n')
            f.write('    #include "${WM_PROJECT_DIR}/etc/caseDicts/setConstraintTypes"\n')

            f.write('    ".*"\n')
            f.write('    {\n')
            f.write('        type            MarshakRadiation;\n')
            f.write('        T               T;\n')
            f.write('        emissivityMode  lookup;\n')
            f.write('        emissivity      uniform 1.0;\n')
            f.write('        value           uniform 0;\n')
            f.write('    }\n')
	    for j in range(len(bcName)):
		if bctype[j]=='wall':
		    f.write('    '+bcName[j]+'\n')
                    f.write('    {\n')
            	    f.write('        type            MarshakRadiation;\n')
            	    f.write('        T               T;\n')
            	    f.write('        emissivityMode  lookup;\n')
            	    f.write('        emissivity      '+emissivities[bcName[j]]+';\n')
            	    f.write('        value           uniform 0;\n')
            	    f.write('    }\n')
            f.write('}\n')
	    f.close()
    #------------------------------------------------------------------------------
    def IDefaultFile(self,emissivities):
	fcom=self.foamComment()
	head=self.BCheader('volScalarField','IDefault')
	hhh=fcom+head

	for i in range(1):
	    bcName=self.getBCName();bctype=self.getBCType()

            f=open(self.caseDir+'/0/IDefault','w')
	    for aaa in hhh:f.write(aaa)	
            f.write('dimensions      [1 0 -3 0 0 0 0];\n')
            f.write('internalField   uniform 0;\n')
            f.write('boundaryField\n')
            f.write('{\n')
            f.write('    #include "${WM_PROJECT_DIR}/etc/caseDicts/setConstraintTypes"\n')

            f.write('    ".*"\n')
            f.write('    {\n')
            f.write('        type            greyDiffusiveRadiation;\n')
            f.write('        T               T;\n')
            f.write('        emissivityMode  lookup;\n')
            f.write('        emissivity      uniform 1.0;\n')
            f.write('        value           uniform 0;\n')
            f.write('    }\n')
	    for j in range(len(bcName)):
		if bctype[j]=='wall':
		    f.write('    '+bcName[j]+'\n')
                    f.write('    {\n')
            	    f.write('        type            greyDiffusiveRadiation;\n')
            	    f.write('        T               T;\n')
            	    f.write('        emissivityMode  lookup;\n')
            	    f.write('        emissivity      '+emissivities[bcName[j]]+';\n')
            	    f.write('        value           uniform 0;\n')
            	    f.write('    }\n')
            f.write('}\n')
	    f.close()
    #------------------------------------------------------------------------------
    def radiationProperties(self,radprop):
	radmodel=radprop[0];freq=radprop[1];abso=radprop[2];emis=radprop[3];ee=radprop[4]
	nphi=radprop[5];ntheta=radprop[6];conv=radprop[7];maxi=radprop[8];cache=radprop[9]
	fcom=self.foamComment()
	head=self.header('constant','radiationProperties')
	hhh=fcom+head
	for i in range(1):
            f=open(self.caseDir+'/constant/radiationProperties','w')
	    for aaa in hhh:f.write(aaa)	

	    f.write('radiation on;\n')
	    f.write('radiationModel  '+radmodel+';\n')
	    f.write('fvDOMCoeffs\n')
	    f.write('{\n')
	    f.write('    nPhi        '+nphi+';\n')
	    f.write('    nTheta      '+ntheta+';\n')
	    f.write('    convergence '+conv+';\n')
	    f.write('    maxIter     '+maxi+';\n')  
	    f.write('    cacheDiv    '+cache+';\n')
	    f.write('}\n')
	    f.write('solverFreq '+freq+';\n')
	    f.write('absorptionEmissionModel constantAbsorptionEmission;\n')
	    f.write('constantAbsorptionEmissionCoeffs\n')
	    f.write('{\n')
	    f.write('   absorptivity    absorptivity    [ m^-1 ]         '+abso+';\n')
	    f.write('   emissivity      emissivity      [ m^-1 ]         '+emis+';\n')
	    f.write('   E               E               [ kg m^-1 s^-3 ] '+ee+';\n')
	    f.write('}\n')
	    f.write('scatterModel    none;\n')
	    f.write('sootModel       none;\n')
	    f.close()
    #------------------------------------------------------------------------------
    def RASProperties(self,turModel):
	fcom=self.foamComment()
	head=self.header('constant','RASProperties')
	hhh=fcom+head
	for i in range(1):
            f=open(self.caseDir+'/constant/RASProperties','w')
	    for aaa in hhh:f.write(aaa)	
            f.write('RASModel        '+turModel+';\n')
            f.write('turbulence      on;\n')
            f.write('printCoeffs     on;\n')
            f.close()
    #------------------------------------------------------------------------------
    def turbulenceProperties(self,turModel):
	fcom=self.foamComment()
	head=self.header('constant','turbulenceProperties')
	hhh=fcom+head

        f=open(self.caseDir+'/constant/turbulenceProperties','w')
	for aaa in hhh:f.write(aaa)	
	if turModel=='laminar':f.write('simulationType        laminar;\n')
	else:f.write('simulationType        RASModel;\n')
        f.close()
    #------------------------------------------------------------------------------
    def gg(self,gravity):

	if gravity=='off':gr='(0 0 0)'
	elif gravity=='x':gr='(1 0 0)'
	elif gravity=='y':gr='(0 1 0)'
	elif gravity=='z':gr='(0 0 1)'
	elif gravity=='-x':gr='(-1 0 0)'
	elif gravity=='-y':gr='(0 -1 0)'
	elif gravity=='-z':gr='(0 0 -1)'

	fcom=self.foamComment()
	head=self.header('constant','g')
	hhh=fcom+head
        f=open(self.caseDir+'/constant/g','w')
	for aaa in hhh:f.write(aaa)
        f.write('dimensions      [0 1 -2 0 0 0 0];\n')
        f.write('value           '+gr+';\n')
        f.close()
    #------------------------------------------------------------------------------
    def operatingPressure(self,op):

	if op=='none':op='101325'

	fcom=self.foamComment()
	head=self.header('constant','operatingConditions')
	hhh=fcom+head
        f=open(self.caseDir+'/constant/operatingConditions','w')
	for aaa in hhh:f.write(aaa)
        f.write('Op              Op [1 -1 -2 0 0 0 0] '+op+';\n')
        f.close()
    #------------------------------------------------------------------------------
    def transportProperties(self,mu,rho):

	if mu[:10]=='Sutherland' or rho=='perfectGas':nu='1e-5'
	else:nu=str(float(mu)/float(rho))

	fcom=self.foamComment()
	head=self.header('constant','transportProperties')
	hhh=fcom+head

        f=open(self.caseDir+'/constant/transportProperties','w')
	for aaa in hhh:f.write(aaa)
	f.write('transportModel    Newtonian;\n')
	f.write('nu    nu [ 0 2 -1 0 0 0 0 ] '+nu+';\n')
	f.close()
    #------------------------------------------------------------------------------
    def dynamicMeshDict(self,sm):

	fcom=self.foamComment()
	head=self.header('constant','dynamicMeshDict')
	hhh=fcom+head

        f=open(self.caseDir+'/constant/dynamicMeshDict','w')
	for aaa in hhh:f.write(aaa)
	f.write('dynamicFvMesh   multiSolidBodyMotionFvMesh;\n')
	f.write('motionSolverLibs ( "libfvMotionSolvers.so" );\n')
	f.write('\n')
	f.write('multiSolidBodyMotionFvMeshCoeffs\n')
	f.write('{\n')
	for ii in sm:f.write('    #include "'+ii+'"\n')
	f.write('}\n')
        f.close()
    #------------------------------------------------------------------------------
    def thermophysicalProperties(self,rho,mu,cp,kappa):

	if mu[:10]=='Sutherland':
	    aa=mu[:-1].split(',');As=aa[1];Ts=aa[2]
	else:
	    pr=str(float(cp)*float(mu)/float(kappa))

	fcom=self.foamComment()
	head=self.header('constant','thermophysicalProperties')
	hhh=fcom+head

        f=open(self.caseDir+'/constant/thermophysicalProperties','w')
	for aaa in hhh:f.write(aaa)

	if rho!='perfectGas':	
            f.write('thermoType\n')
            f.write('{\n')
	    kk=['type','mixture','transport','thermo','equationOfState','specie','energy']
	    if mu[:10]=='Sutherland':
	        vv=['heRhoThermo','pureMixture','sutherland','hConst','rhoConst','specie','sensibleEnthalpy']
	    else:
	        vv=['heRhoThermo','pureMixture','const','hConst','rhoConst','specie','sensibleEnthalpy']
	    for i in range(len(kk)):f.write('    '+kk[i]+'		'+vv[i]+';\n')
            f.write('}\n')
            f.write('mixture\n')
            f.write('{\n')
            f.write('    specie\n')
            f.write('    {\n')
            f.write('        nMoles          1;\n')
            f.write('        molWeight       28.9;\n')
            f.write('    }\n')
            f.write('    thermodynamics\n')
            f.write('    {\n')
            f.write('        Cp              '+cp+';\n')
            f.write('        Hf              0;\n')
            f.write('    }\n')
            f.write('    transport\n')
            f.write('    {\n')
	    if mu[:10]=='Sutherland':
                f.write('        As              '+As+';\n')
                f.write('        Ts              '+Ts+';\n')
	    else:
                f.write('        mu              '+mu+';\n')
                f.write('        Pr              '+pr+';\n')
            f.write('    }\n')
            f.write('    equationOfState\n')
            f.write('    {\n')
            f.write('        rho             '+rho+';\n')
            f.write('    }\n')
            f.write('}\n')
	else:
            f.write('thermoType\n')
            f.write('{\n')
	    kk=['type','mixture','transport','thermo','equationOfState','specie','energy']
	    if mu[:10]=='Sutherland':
	        vv=['heRhoThermo','pureMixture','sutherland','hConst','perfectGas','specie','sensibleEnthalpy']
	    else:
	        vv=['heRhoThermo','pureMixture','const','hConst','perfectGas','specie','sensibleEnthalpy']
	    for i in range(len(kk)):f.write('    '+kk[i]+'		'+vv[i]+';\n')
            f.write('}\n')
            f.write('mixture\n')
            f.write('{\n')
            f.write('    specie\n')
            f.write('    {\n')
            f.write('        nMoles          1;\n')
            f.write('        molWeight       28.9;\n')
            f.write('    }\n')
            f.write('    thermodynamics\n')
            f.write('    {\n')
            f.write('        Cp              '+cp+';\n')
            f.write('        Hf              0;\n')
            f.write('    }\n')
            f.write('    transport\n')
            f.write('    {\n')
	    if mu[:10]=='Sutherland':
                f.write('        As              '+As+';\n')
                f.write('        Ts              '+Ts+';\n')
	    else:
                f.write('        mu              '+mu+';\n')
                f.write('        Pr              '+pr+';\n')
            f.write('    }\n')
            f.write('}\n')
	f.close()
    #------------------------------------------------------------------------------
    def decomposeParDict(self,ncore):
	fcom=self.foamComment()
	head=self.header('system','decomposeParDict')
	hhh=fcom+head
        f=open(self.caseDir+'/system/decomposeParDict','w')
	for aaa in hhh:f.write(aaa)
        f.write('numberOfSubdomains     '+ncore+';\n')
        f.write('method                 scotch;\n')
        f.close()
    #------------------------------------------------------------------------------
    def fvSchemes(self,fvSchemesList):
	#fvSchemesList=[ddtScheme,gradScheme,laplacianScheme,interpolationScheme,snGrad,divU,divk,divh,solver]
	ddtSchemes=fvSchemesList[0];gradScheme=fvSchemesList[1];laplacianScheme=fvSchemesList[2]
	interpolationScheme=fvSchemesList[3];snGrad=fvSchemesList[4];divU=fvSchemesList[5];divk=fvSchemesList[6]
	divh=fvSchemesList[7];solver=fvSchemesList[8]

	fcom=self.foamComment()
	head=self.header('system','fvSchemes')
	hhh=fcom+head

	for i in range(1):

            f=open(self.caseDir+'/system/fvSchemes','w')
	    for aaa in hhh:f.write(aaa)

            f.write('ddtSchemes\n')
            f.write('{\n')
	    if solver=='buoyantSimpleFoam':
                f.write('    default         steadyState;\n')
	    else:
                f.write('    default         '+ddtSchemes+';\n')
            f.write('}\n')
            f.write('gradSchemes\n')
            f.write('{\n')
            f.write('    default         '+gradScheme+';\n')
            f.write('}\n')
            f.write('divSchemes\n')
            f.write('{\n')
            f.write('    default          Gauss linear;\n')
            f.write('    div(phi,U)       '+divU+';\n')
            f.write('    div(phi,K)       Gauss upwind;\n')
            f.write('    div(phi,h)       '+divh+';\n')
            f.write('    div(phi,k)       '+divk+';\n')
            f.write('    div(phi,epsilon) '+divk+';\n')
            f.write('    div(phi,omega)   '+divk+';\n')
            f.write('    div((nuEff*dev(T(grad(U))))) Gauss linear;\n')
            f.write('    div((nuEff*T(grad(U)))) Gauss linear;\n')
            f.write('    div((muEff*dev2(T(grad(U))))) Gauss linear;\n')
            f.write('    div(Ji,Ii_h) Gauss linearUpwind grad(Ii_h);\n')
            f.write('}\n')
            f.write('laplacianSchemes\n')
            f.write('{\n')
            f.write('    default         '+laplacianScheme+';\n')
            f.write('}\n')
            f.write('interpolationSchemes\n')
            f.write('{\n')
            f.write('    default         '+interpolationScheme+';\n')
            f.write('}\n')
            f.write('snGradSchemes\n')
            f.write('{\n')
            f.write('    default         '+snGrad+';\n')
            f.write('}\n')
            f.write('fluxRequired\n')
            f.write('{\n')
            f.write('    default         no;\n')
            f.write('    p_rgh             ;\n')
            f.write('    pcorr             ;\n')
            f.write('    p                 ;\n')
            f.write('}\n')
            f.close()
    #------------------------------------------------------------------------------
    def fvSolution(self,fvSolutionList):
	#fvSolutionList=[pSolver,USolver,hSolver,momentumPredictor,nOuterCorrectors,nCorrectors,nonOrthogonalCorrectors,
	#		relaxationp,relaxationU,relaxationk,relaxationh]
	pSolver=fvSolutionList[0];USolver=fvSolutionList[1];hSolver=fvSolutionList[2];momentumPredictor=fvSolutionList[3]
	nOuterCorrectors=fvSolutionList[4];nCorrectors=fvSolutionList[5];nonOrthogonalCorrectors=fvSolutionList[6]
	relaxationp=fvSolutionList[7];relaxationU=fvSolutionList[8];relaxationk=fvSolutionList[9];relaxationh=fvSolutionList[10]

	fcom=self.foamComment()
	head=self.header('system','fvSolution')
	hhh=fcom+head

	for i in range(1):

            f=open(self.caseDir+'/system/fvSolution','w')
	    for aaa in hhh:f.write(aaa)

	    kkGAMG=['solver               ','tolerance            ','relTol               ','smoother             ',
		    'nPreSweeps           ','nPostSweeps          ','cacheAgglomeration   ','agglomerator         ',
		    'nCellsInCoarsestLevel','mergeLevels          ']
	    vvGAMG=['GAMG','1e-06','0.01','GaussSeidel','0','2','on','faceAreaPair','10','1']
	    kkPCG=['solver              ','preconditioner       ','tolerance            ','relTol               ']
	    vvPCG=['PCG','DIC','1e-6','0.01']
	    kkPBiCG=['solver         ','preconditioner ','tolerance      ','relTol         ']
	    vvPBiCG=['PBiCG','DILU','1e-6','0.01']
	    kksmoothSolver=['solver         ','smoother       ','tolerance      ','relTol         ','minIter        ']
	    vvsmoothSolver=['smoothSolver','symGaussSeidel','1e-6','0.01','1']

            f.write('solvers\n')
            f.write('{\n')
            f.write('    rho\n')
            f.write('    {\n')
            f.write('        solver                  GAMG;\n')
            f.write('        smoother                GaussSeidel;\n')
            f.write('        nPreSweeps              0;\n')
            f.write('        nPostSweeps             2;\n')
            f.write('        cacheAgglomeration      true;\n')
            f.write('        nCellsInCoarsestLevel   10;\n')
            f.write('        agglomerator            faceAreaPair;\n')
            f.write('        mergeLevels             1;\n')
            f.write('        tolerance               1e-12;\n')
            f.write('        relTol                  0.1;\n')
            f.write('    }\n')
            f.write('    rhoFinal\n')
            f.write('    {\n')
            f.write('        $rho                    ;\n')
            f.write('        tolerance               1e-06;\n')
            f.write('        relTol                  0;\n')
            f.write('    }\n')

            f.write('    p_rgh\n')
            f.write('    {\n')
	    if pSolver=='GAMG':
	        for i in range(len(kkGAMG)):f.write('        '+kkGAMG[i]+'    '+vvGAMG[i]+';\n')
	    elif pSolver=='PCG':
	        for i in range(len(kkPCG)):f.write('        '+kkPCG[i]+'    '+vvPCG[i]+';\n')
            f.write('    }\n')

            f.write('    p_rghFinal\n')
            f.write('    {\n')
	    f.write('        $p_rgh;\n')
	    f.write('        relTol    0;\n')
            f.write('    }\n')

            f.write('    "(U|k|omega|epsilon)"\n')
            f.write('    {\n')
	    if USolver=='smoothSolver':
	        for i in range(len(kksmoothSolver)):f.write('        '+kksmoothSolver[i]+'    '+vvsmoothSolver[i]+';\n')
	    elif USolver=='PBiCG':
	        for i in range(len(kkPBiCG)):f.write('        '+kkPBiCG[i]+'    '+vvPBiCG[i]+';\n')
            f.write('    }\n')

            f.write('    "(U|k|omega|epsilon)Final"\n')
            f.write('    {\n')
	    f.write('        $U;\n')
	    f.write('        relTol    0;\n')
            f.write('    }\n')


            f.write('    h\n')
            f.write('    {\n')
	    if hSolver=='smoothSolver':
	        for i in range(len(kksmoothSolver)):f.write('        '+kksmoothSolver[i]+'    '+vvsmoothSolver[i]+';\n')
	    elif hSolver=='PBiCG':
	        for i in range(len(kkPBiCG)):f.write('        '+kkPBiCG[i]+'    '+vvPBiCG[i]+';\n')
            f.write('    }\n')

            f.write('    hFinal\n')
            f.write('    {\n')
	    f.write('        $h;\n')
	    f.write('        relTol    0;\n')
            f.write('    }\n')

            f.write('    Ii\n')
            f.write('    {\n')
            f.write('        solver              GAMG;\n')
            f.write('        tolerance           1e-4;\n')
            f.write('        relTol              0;\n')
            f.write('        smoother            symGaussSeidel;\n')
            f.write('        cacheAgglomeration  true;\n')
            f.write('        nCellsInCoarsestLevel   10;\n')
            f.write('        agglomerator        faceAreaPair;\n')
            f.write('        mergeLevels         1;\n')
            f.write('        maxIter             5;\n')
            f.write('        nPreSweeps          0;\n')
            f.write('        nPostSweeps         1;\n')
            f.write('    }\n')

            f.write('    G\n')
            f.write('    {\n')
            f.write('        solver          PCG;\n')
            f.write('        preconditioner  DIC;\n')
            f.write('        tolerance       1e-06;\n')
            f.write('        relTol          0.01;\n')
            f.write('    }\n')


            f.write('    pcorr\n')
            f.write('    {\n')
	    if pSolver=='GAMG':
	        for i in range(len(kkGAMG)):f.write('        '+kkGAMG[i]+'    '+vvGAMG[i]+';\n')
	    elif pSolver=='PCG':
	        for i in range(len(kkPCG)):f.write('        '+kkPCG[i]+'    '+vvPCG[i]+';\n')
            f.write('    }\n')

            f.write('    p\n')
            f.write('    {\n')
	    f.write('        $pcorr;\n')
            f.write('    }\n')

            f.write('    pFinal\n')
            f.write('    {\n')
	    f.write('        $pcorr;\n')
	    f.write('        relTol    0;\n')
            f.write('    }\n')	    

            f.write('}\n')

            f.write('SIMPLE\n')
            f.write('{\n')
            f.write('    nNonOrthogonalCorrectors  '+nonOrthogonalCorrectors+';\n')
            f.write('    pRefCell                  0;\n')
            f.write('    pRefValue                 0;\n')
            f.write('    residualControl\n')
            f.write('    {\n')
            f.write('        p                    1e-6;\n')
            f.write('        U                    1e-6;\n')
            f.write('        "(k|epsilon|omega)"  1e-6;\n')
            f.write('    }\n')
            f.write('}\n')

            f.write('potentialFlow\n')
            f.write('{\n')
            f.write('    nNonOrthogonalCorrectors 10;\n')
            f.write('    pRefCell                  0;\n')
            f.write('    pRefValue                 0;\n')
            f.write('}\n')

            f.write('PIMPLE\n')
            f.write('{\n')
            f.write('    correctPhi                yes;\n')
            f.write('    momentumPredictor         '+momentumPredictor+';\n')
            f.write('    nOuterCorrectors          '+nOuterCorrectors+';\n')
            f.write('    nCorrectors               '+nCorrectors+';\n')
            f.write('    nNonOrthogonalCorrectors  '+nonOrthogonalCorrectors+';\n')
            f.write('    pRefCell                  0;\n')
            f.write('    pRefValue                 0;\n')
            f.write('}\n')

            f.write('relaxationFactors\n')
            f.write('{\n')
            f.write('    fields\n')
            f.write('    {\n')
            f.write('        "rho.*"              1.0;\n')
            f.write('        "p_rgh.*"            '+relaxationp+';\n')
            f.write('    }\n')
            f.write('    equations\n')
            f.write('    {\n')
            f.write('        "U.*"                '+relaxationU+';\n')
            f.write('        "(k|epsilon|omega).*"  '+relaxationk+';\n')
            f.write('        "h.*"                '+relaxationh+';\n')
            f.write('        "ILambda.*"          1.0;\n')
            f.write('        "G.*"                1.0;\n')
            f.write('    }\n')
            f.write('}\n')
            f.close()
    #------------------------------------------------------------------------------
    def controlDict(self,controlDictList):
	#controlDictList=[startFrom,startTime,stopAt,endTime,deltaT,adjustTimeStep,maxCo,maxDeltaT,writeControl,
	#		  writeInterval,purgeWrite,writeFormat,writePrecision,writeCompression,solver]
	f1=glob.glob(self.caseDir+'/system/probeFunction')
	f2=glob.glob(self.caseDir+'/system/surfaceProbeFunction')
	f3=glob.glob(self.caseDir+'/system/flowRateFunction')
	f4=glob.glob(self.caseDir+'/system/forces')
	f5=glob.glob(self.caseDir+'/system/forceCoeffs')

	fcom=self.foamComment()
	head=self.header('system','controlDict')
	hhh=fcom+head
        f=open(self.caseDir+'/system/controlDict','w')
	for aaa in hhh:f.write(aaa)
	f.write('libs		("libturbulenceDerivedFvPatchFields.so");\n')
	aa=controlDictList
	ckk=['application      ','startFrom        ','startTime        ','stopAt           ','endTime          ',
	     'deltaT           ','writeControl     ','writeInterval    ','purgeWrite       ','writeFormat      ',
	     'writePrecision   ','writeCompression ','timeFormat       ','timePrecision    ','runTimeModifiable',
	     'adjustTimeStep   ','maxCo            ','maxDeltaT        ']
	cvv=[aa[14],aa[0],aa[1],aa[2],aa[3],aa[4],aa[8],aa[9],aa[10],aa[11],aa[12],aa[13],'general','6','true',
	     aa[5],aa[6],aa[7]]
	if aa[14]=='simpleFoam':cvv[5]='1'

	for i in range(len(ckk)):f.write(ckk[i]+'    '+cvv[i]+';\n')

        f.write('functions\n')
        f.write('{\n')
	if len(f1)!=0:f.write('    #include "probeFunction"\n')
	if len(f2)!=0:f.write('    #include "surfaceProbeFunction"\n')
	if len(f3)!=0:f.write('    #include "flowRateFunction"\n')
	if len(f4)!=0:f.write('    #include "forces"\n')
	if len(f5)!=0:f.write('    #include "forceCoeffs"\n')
        f.write('}\n')
        f.close()
    #------------------------------------------------------------------------------
    def customRegexp(self):
        f=open(self.caseDir+'/customRegexp','w')
        f.write('Moment\n')
        f.write('{\n')
        f.write('	expr "Cd    = (%f%)";\n')
        f.write('	name	Cd;\n')
        f.write('	theTitle	"drag coeffs.";\n')
        f.write('	titles	("Cd");\n')
        f.write('	type	regular;\n')
        f.write('}\n')
        f.close()
    #------------------------------------------------------------------------------
    def fvOptions(self,po,mr,fu,ft,ms,es):
	fcom=self.foamComment()
	head=self.header('system','fvOptions')
	hhh=fcom+head
	for i in range(1):
            f=open(self.caseDir+'/system/fvOptions','w')
	    for aaa in hhh:f.write(aaa)
	    if len(po)!=0:
		for ii in po:f.write('#include "'+ii+'"\n')
	    if len(mr)!=0:
		for ii in mr:f.write('#include "'+ii+'"\n')
	    if len(fu)!=0:
		for ii in fu:f.write('#include "'+ii+'"\n')
	    if len(ms)!=0:
		for ii in ms:f.write('#include "'+ii+'"\n')
	    if len(ft)!=0:
		for ii in ft:f.write('#include "'+ii+'"\n')
	    if len(es)!=0:
		for ii in es:f.write('#include "'+ii+'"\n')
            f.close()
    #------------------------------------------------------------------------------
    def force(self,faces):
        f=open(self.caseDir+'/system/forces','w')

	f.write('forces\n')
	f.write('{\n')
	f.write('    type        	forces;\n')
	f.write('    functionObjectLibs ( "libforces.so" );\n')
	f.write('    outputControl   	timeStep;\n')
	f.write('    timeInterval   	1;\n')
	f.write('    log         	yes;\n')
	f.write('    patches \n')
	f.write('    ( \n')
	for ii in faces:f.write('        '+ii+' \n')
	f.write('    ); \n')
	f.write('    pName       	p;\n')
	f.write('    UName       	U;\n')
	f.write('    rhoName     	rhoInf; \n')
	f.write('    log         	true;\n')
	f.write('    rhoInf      	1000;  \n')
	f.write('    liftDir     	(0 1 0);\n')
	f.write('    dragDir     	(1 0 0);\n')
	f.write('    CofR        	(0 0 0.05);\n')
	f.write('    pitchAxis   	(0 0 1);\n')
	f.write('    magUInf     	1;\n')
	f.write('    lRef        	1;\n')    
	f.write('    Aref        	1;\n')   
	f.write('}\n')
	f.close()

        f=open(self.caseDir+'/system/forceCoeffs','w')

	f.write('forceCoeffs\n')
	f.write('{\n')
	f.write('    type        	forceCoeffs;\n')
	f.write('    functionObjectLibs ( "libforces.so" );\n')
	f.write('    outputControl   	timeStep;\n')
	f.write('    timeInterval   	1;\n')
	f.write('    log         	yes;\n')
	f.write('    patches \n')
	f.write('    ( \n')
	for ii in faces:f.write('        '+ii+' \n')
	f.write('    ); \n')
	f.write('    pName       	p;\n')
	f.write('    UName       	U;\n')
	f.write('    rhoName     	rhoInf; \n')
	f.write('    log         	true;\n')
	f.write('    rhoInf      	1000;  \n')
	f.write('    liftDir     	(0 1 0);\n')
	f.write('    dragDir     	(1 0 0);\n')
	f.write('    CofR        	(0 0 0.05);\n')
	f.write('    pitchAxis   	(0 0 1);\n')
	f.write('    magUInf     	1;\n')
	f.write('    lRef        	1;        \n')
	f.write('    Aref        	1;       \n')
	f.write('}\n')
	f.close()
    #-----------------------------------------------------------------------------------------

    #------------------------------------------------------------------------------
    #    create baffle section
    #------------------------------------------------------------------------------

    def topoSetDict(self,sets):
	fcom=self.foamComment()
	head=self.header('system','topoSetDict')
	hhh=fcom+head
        f=open(self.caseDir+'/system/topoSetDict','w')
	for aaa in hhh:f.write(aaa)
	f.write('actions\n')
	f.write('(\n')
	for ss in sets:
	    f.write('    {\n')
	    f.write('        name    '+ss+';\n')
	    f.write('        type    faceZoneSet;\n')
	    f.write('        action  new;\n')
	    f.write('        source  setToFaceZone;\n')
	    f.write('        sourceInfo\n')
	    f.write('        {\n')
	    f.write('            faceSet '+ss+';\n')
	    f.write('        }\n')
	    f.write('    }\n')
	f.write(');\n')
	f.close()  
    #------------------------------------------------------------------------------
    def createBafflesDict(self,sets):
	fcom=self.foamComment()
	head=self.header('system','createBafflesDict')
	hhh=fcom+head
        f=open(self.caseDir+'/system/createBafflesDict','w')
	for aaa in hhh:f.write(aaa)
	f.write('internalFacesOnly true;\n')
	f.write('baffles\n')
	f.write('{\n')
	for ss in sets:
	    f.write('    '+ss+'\n')
	    f.write('    {\n')
	    f.write('        type        faceZone;\n')
	    f.write('        zoneName    '+ss+';\n')
	    f.write('        patchPairs\n')
	    f.write('        {\n')
	    f.write('            type	wall;\n')
	    f.write('	     }\n')
	    f.write('    }\n')
	f.write('}\n')
	f.close()
    #------------------------------------------------------------------------------
    def createBafflesDictThermal(self,sets):
	fcom=self.foamComment()
	head=self.header('system','createBafflesDict')
	hhh=fcom+head
        f=open(self.caseDir+'/system/createBafflesDict','w')
	for aaa in hhh:f.write(aaa)
	f.write('internalFacesOnly true;\n')
	f.write('baffles\n')
	f.write('{\n')
	for ss in sets:
	    f.write('    '+ss+'\n')
	    f.write('    {\n')
	    f.write('        type        faceZone;\n')
	    f.write('        zoneName    '+ss+';\n')
	    f.write('        patchPairs\n')
	    f.write('        {\n')
	    f.write('            type	mappedWall;\n')
	    f.write('            sampleMode	nearestPatchFace;\n')
	    f.write('	     }\n')
	    f.write('    }\n')
	f.write('}\n')
	f.close()
    #------------------------------------------------------------------------------
    def createBafflesDictInterior(self,sets):
	fcom=self.foamComment()
	head=self.header('system','createBafflesDict')
	hhh=fcom+head
        f=open(self.caseDir+'/system/createBafflesDict','w')
	for aaa in hhh:f.write(aaa)
	f.write('internalFacesOnly true;\n')
	f.write('baffles\n')
	f.write('{\n')
	for ss in sets:
	    f.write('    '+ss+'\n')
	    f.write('    {\n')
	    f.write('        type        faceZone;\n')
	    f.write('        zoneName    '+ss+';\n')
	    f.write('        patches\n')
	    f.write('        {\n')
	    f.write('            master\n')
	    f.write('            {\n')
	    f.write('                name        '+ss+'_master;\n')
	    f.write('                type        cyclic;\n')
	    f.write('                matchTolerance  0.00001;\n')
	    f.write('                neighbourPatch  '+ss+'_slave;\n')
	    f.write('            }\n')
	    f.write('            slave\n')
	    f.write('            {\n')
	    f.write('                name        '+ss+'_slave;\n')
	    f.write('                type        cyclic;\n')
	    f.write('                matchTolerance  0.00001;\n')
	    f.write('                neighbourPatch  '+ss+'_master;\n')
	    f.write('            }\n')
	    f.write('        }\n')
	    f.write('    }\n')
	f.write('}\n')
	f.close()
    #------------------------------------------------------------------------------
    #    create AMI section
    #------------------------------------------------------------------------------
    def createAMIFileTransition(self,sets,cc1,cc2,cc3):
	#if axis=='x':raxis='(1 0 0)'
	#elif axis=='y':raxis='(0 1 0)'
	#elif axis=='z':raxis='(0 0 1)'

	vector1='('+cc1+' '+cc2+' '+cc3+')'
	xx=-1*float(cc1);yy=-1*float(cc2);zz=-1*float(cc3)
	vector2='('+str(xx)+' '+str(yy)+' '+str(zz)+')'

	fcom=self.foamComment()
	head=self.header('system','changeDictionaryDict')
	hhh=fcom+head
        f=open(self.caseDir+'/system/changeDictionaryDictCyclic','w')
	for aaa in hhh:f.write(aaa)
        f.write('dictionaryReplacement\n')
        f.write('{\n')
        f.write('    boundary\n')
        f.write('    {\n')
        f.write('        '+sets[0]+'\n')
        f.write('        {\n')
        f.write('            type              cyclicAMI;\n')
        f.write('            matchTolerance    0.0001;\n')
        f.write('            transform         translational;\n')
        f.write('            neighbourPatch    '+sets[1]+';\n')
        f.write('            separationVector  '+vector1+';\n')
        f.write('        }\n')
        f.write('        '+sets[1]+'\n')
        f.write('        {\n')
        f.write('            type              cyclicAMI;\n')
        f.write('            matchTolerance    0.0001;\n')
        f.write('            transform         translational;\n')
        f.write('            neighbourPatch    '+sets[0]+';\n')
        f.write('            separationVector  '+vector2+';\n')
        f.write('        }\n')
        f.write('    }\n')
        f.write('}\n')
        f.close() 
    #------------------------------------------------------------------------------
    def createAMIFile(self,sets,cc1,cc2,cc3,axis):
	if axis=='x':raxis='(1 0 0)'
	elif axis=='y':raxis='(0 1 0)'
	elif axis=='z':raxis='(0 0 1)'

	fcom=self.foamComment()
	head=self.header('system','changeDictionaryDict')
	hhh=fcom+head
        f=open(self.caseDir+'/system/changeDictionaryDictAMI','w')
	for aaa in hhh:f.write(aaa)
        f.write('dictionaryReplacement\n')
        f.write('{\n')
        f.write('    boundary\n')
        f.write('    {\n')
        f.write('        '+sets[0]+'\n')
        f.write('        {\n')
        f.write('            type              cyclicAMI;\n')
        f.write('            matchTolerance    0.0001;\n')
        f.write('            transform         rotational;\n')
        f.write('            neighbourPatch    '+sets[1]+';\n')
        f.write('            rotationAxis      '+raxis+';\n')
        f.write('            rotationCentre    ('+cc1+' '+cc2+' '+cc3+');\n')
        f.write('        }\n')
        f.write('        '+sets[1]+'\n')
        f.write('        {\n')
        f.write('            type              cyclicAMI;\n')
        f.write('            matchTolerance    0.0001;\n')
        f.write('            transform         rotational;\n')
        f.write('            neighbourPatch    '+sets[0]+';\n')
        f.write('            rotationAxis      '+raxis+';\n')
        f.write('            rotationCentre    ('+cc1+' '+cc2+' '+cc3+');\n')
        f.write('        }\n')
        f.write('    }\n')
        f.write('}\n')
        f.close() 
    #------------------------------------------------------------------------------
    def setFieldsDict(self,setFieldsRangeMin,setFieldsRangeMax,scalar,defualtValue,value):
	
	fcom=self.foamComment()
	head=self.header('system','setFieldsDict')
	hhh=fcom+head
        f=open(self.caseDir+'/system/setFieldsDict','w')
	for aaa in hhh:f.write(aaa)

	f.write('defaultFieldValues\n')
	f.write('(\n')
	f.write('    volScalarFieldValue '+scalar+' '+defualtValue+' \n')
	f.write(');\n')
	f.write('regions\n')
	f.write('(\n')
	f.write('    boxToCell\n')
	f.write('    {\n')
	f.write('        box '+setFieldsRangeMin+' '+setFieldsRangeMax+';\n')
	f.write('        fieldValues\n')
	f.write('        (\n')
	f.write('            volScalarFieldValue '+scalar+' '+value+'\n')
	f.write('        );\n')
	f.write('    }\n')
	f.write(');\n')
	f.close()
    #------------------------------------------------------------------------------
    def UFileBasic(self):
	fcom=self.foamComment();head=self.BCheader('volVectorField','U');hhh=fcom+head

	for i in range(1):
	    bcname=self.getBCName();bctype=self.getBCType()
            f=open(self.caseDir+'/0/U','w')
	    for aaa in hhh:f.write(aaa)	
            f.write('dimensions      [0 1 -1 0 0 0 0];\n')
            f.write('internalField   uniform (0 0 0);\n')
            f.write('boundaryField\n')
            f.write('{\n')
            #f.write('    #include "${WM_PROJECT_DIR}/etc/caseDicts/setConstraintTypes"\n')
	    for j in range(len(bcname)):
		f.write('    '+bcname[j]+'\n')
                f.write('    {\n')
		if bctype[j]=='wall' or bctype[j]=='mappedWall':
            	    f.write('        type    fixedValue;\n')
            	    f.write('        value   uniform (0 0 0);\n')
		elif bctype[j]=='patch':
            	    f.write('        type    fixedValue;\n')
            	    f.write('        value   uniform (0 0 0);\n')
		elif bctype[j]=='symmetry':
            	    f.write('        type    symmetry;\n')
		elif bctype[j]=='symmetryPlane':
            	    f.write('        type    symmetryPlane;\n')
		elif bctype[j]=='empty':
            	    f.write('        type    empty;\n')
		elif bctype[j]=='cyclic':
            	    f.write('        type    cyclic;\n')
		elif bctype[j]=='cyclicAMI':
            	    f.write('        type    cyclicAMI;\n')
       	        f.write('    }\n')
            f.write('}\n')
	    f.close()
    #------------------------------------------------------------------------------
    def pFileBasic(self):
	fcom=self.foamComment();head=self.BCheader('volScalarField','p');hhh=fcom+head

	for i in range(1):
	    bcname=self.getBCName();bctype=self.getBCType()
            f=open(self.caseDir+'/0/p','w')
	    for aaa in hhh:f.write(aaa)	
            f.write('dimensions      [1 -1 -2 0 0 0 0];\n')
            f.write('internalField   uniform 101325;\n')
            f.write('boundaryField\n')
            f.write('{\n')
            #f.write('    #include "${WM_PROJECT_DIR}/etc/caseDicts/setConstraintTypes"\n')
	    for j in range(len(bcname)):
		f.write('    '+bcname[j]+'\n')
                f.write('    {\n')
		if bctype[j]=='wall' or bctype[j]=='mappedWall':
            	    f.write('        type    calculated;\n')
		elif bctype[j]=='patch':
            	    f.write('        type    calculated;\n')
		elif bctype[j]=='symmetry':
            	    f.write('        type    symmetry;\n')
		elif bctype[j]=='symmetryPlane':
            	    f.write('        type    symmetryPlane;\n')
		elif bctype[j]=='empty':
            	    f.write('        type    empty;\n')
		elif bctype[j]=='cyclic':
            	    f.write('        type    cyclic;\n')
		elif bctype[j]=='cyclicAMI':
            	    f.write('        type    cyclicAMI;\n')
       	        f.write('    }\n')
            f.write('}\n')
	    f.close()
    #------------------------------------------------------------------------------
    def p_rghFileBasic(self):
	fcom=self.foamComment();head=self.BCheader('volScalarField','p_rgh');hhh=fcom+head

	for i in range(1):
	    bcname=self.getBCName();bctype=self.getBCType()
            f=open(self.caseDir+'/0/p_rgh','w')
	    for aaa in hhh:f.write(aaa)	
            f.write('dimensions      [1 -1 -2 0 0 0 0];\n')
            f.write('internalField   uniform 0;\n')
            f.write('boundaryField\n')
            f.write('{\n')
            #f.write('    #include "${WM_PROJECT_DIR}/etc/caseDicts/setConstraintTypes"\n')
	    for j in range(len(bcname)):
		f.write('    '+bcname[j]+'\n')
                f.write('    {\n')
		if bctype[j]=='wall' or bctype[j]=='mappedWall':
            	    f.write('        type    fixedFluxPressure;\n')
		elif bctype[j]=='patch':
            	    f.write('        type    zeroGradient;\n')
		elif bctype[j]=='symmetry':
            	    f.write('        type    symmetry;\n')
		elif bctype[j]=='symmetryPlane':
            	    f.write('        type    symmetryPlane;\n')
		elif bctype[j]=='empty':
            	    f.write('        type    empty;\n')
		elif bctype[j]=='cyclic':
            	    f.write('        type    cyclic;\n')
		elif bctype[j]=='cyclicAMI':
            	    f.write('        type    cyclicAMI;\n')
       	        f.write('    }\n')
            f.write('}\n')
	    f.close()
    #------------------------------------------------------------------------------
    def TFileBasic(self):
	fcom=self.foamComment();head=self.BCheader('volScalarField','T');hhh=fcom+head

	for i in range(1):
	    bcname=self.getBCName();bctype=self.getBCType()
            f=open(self.caseDir+'/0/T','w')
	    for aaa in hhh:f.write(aaa)	
            f.write('dimensions      [0 0 0 1 0 0 0];\n')
            f.write('internalField   uniform 300;\n')
            f.write('boundaryField\n')
            f.write('{\n')
            #f.write('    #include "${WM_PROJECT_DIR}/etc/caseDicts/setConstraintTypes"\n')
	    for j in range(len(bcname)):
		f.write('    '+bcname[j]+'\n')
                f.write('    {\n')
		if bctype[j]=='wall' or bctype[j]=='mappedWall':
            	    f.write('        type    zeroGradient;\n')
		elif bctype[j]=='patch':
            	    f.write('        type    zeroGradient;\n')
		elif bctype[j]=='symmetry':
            	    f.write('        type    symmetry;\n')
		elif bctype[j]=='symmetryPlane':
            	    f.write('        type    symmetryPlane;\n')
		elif bctype[j]=='empty':
            	    f.write('        type    empty;\n')
		elif bctype[j]=='cyclic':
            	    f.write('        type    cyclic;\n')
		elif bctype[j]=='cyclicAMI':
            	    f.write('        type    cyclicAMI;\n')
       	        f.write('    }\n')
            f.write('}\n')
	    f.close()
    #------------------------------------------------------------------------------
    def kFileBasic(self):
	fcom=self.foamComment();head=self.BCheader('volScalarField','k');hhh=fcom+head

	for i in range(1):
	    bcname=self.getBCName();bctype=self.getBCType()
            f=open(self.caseDir+'/0/k','w')
	    for aaa in hhh:f.write(aaa)	
            f.write('dimensions      [0 2 -2 0 0 0 0];\n')
            f.write('internalField   uniform 1;\n')
            f.write('boundaryField\n')
            f.write('{\n')
            #f.write('    #include "${WM_PROJECT_DIR}/etc/caseDicts/setConstraintTypes"\n')
	    for j in range(len(bcname)):
		f.write('    '+bcname[j]+'\n')
                f.write('    {\n')
		if bctype[j]=='wall' or bctype[j]=='mappedWall':
            	    f.write('        type    compressible::kqRWallFunction;\n')
            	    f.write('        value   $internalField;\n')
		elif bctype[j]=='patch':
            	    f.write('        type    zeroGradient;\n')
		elif bctype[j]=='symmetry':
            	    f.write('        type    symmetry;\n')
		elif bctype[j]=='symmetryPlane':
            	    f.write('        type    symmetryPlane;\n')
		elif bctype[j]=='empty':
            	    f.write('        type    empty;\n')
		elif bctype[j]=='cyclic':
            	    f.write('        type    cyclic;\n')
		elif bctype[j]=='cyclicAMI':
            	    f.write('        type    cyclicAMI;\n')
       	        f.write('    }\n')
            f.write('}\n')
	    f.close()
    #------------------------------------------------------------------------------
    def epsilonFileBasic(self):
	fcom=self.foamComment();head=self.BCheader('volScalarField','epsilon');hhh=fcom+head

	for i in range(1):
	    bcname=self.getBCName();bctype=self.getBCType()
            f=open(self.caseDir+'/0/epsilon','w')
	    for aaa in hhh:f.write(aaa)	
            f.write('dimensions      [0 2 -3 0 0 0 0];\n')
            f.write('internalField   uniform 1;\n')
            f.write('boundaryField\n')
            f.write('{\n')
            #f.write('    #include "${WM_PROJECT_DIR}/etc/caseDicts/setConstraintTypes"\n')
	    for j in range(len(bcname)):
		f.write('    '+bcname[j]+'\n')
                f.write('    {\n')
		if bctype[j]=='wall' or bctype[j]=='mappedWall':
            	    f.write('        type    compressible::epsilonWallFunction;\n')
            	    f.write('        value   $internalField;\n')
		elif bctype[j]=='patch':
            	    f.write('        type    zeroGradient;\n')
		elif bctype[j]=='symmetry':
            	    f.write('        type    symmetry;\n')
		elif bctype[j]=='symmetryPlane':
            	    f.write('        type    symmetryPlane;\n')
		elif bctype[j]=='empty':
            	    f.write('        type    empty;\n')
		elif bctype[j]=='cyclic':
            	    f.write('        type    cyclic;\n')
		elif bctype[j]=='cyclicAMI':
            	    f.write('        type    cyclicAMI;\n')
       	        f.write('    }\n')
            f.write('}\n')
	    f.close()
    #------------------------------------------------------------------------------
    def omegaFileBasic(self):
	fcom=self.foamComment();head=self.BCheader('volScalarField','omega');hhh=fcom+head

	for i in range(1):
	    bcname=self.getBCName();bctype=self.getBCType()
            f=open(self.caseDir+'/0/omega','w')
	    for aaa in hhh:f.write(aaa)	
            f.write('dimensions      [0 0 -1 0 0 0 0];\n')
            f.write('internalField   uniform 1;\n')
            f.write('boundaryField\n')
            f.write('{\n')
            #f.write('    #include "${WM_PROJECT_DIR}/etc/caseDicts/setConstraintTypes"\n')
	    for j in range(len(bcname)):
		f.write('    '+bcname[j]+'\n')
                f.write('    {\n')
		if bctype[j]=='wall' or bctype[j]=='mappedWall':
            	    f.write('        type    compressible::omegaWallFunction;\n')
            	    f.write('        value   $internalField;\n')
		elif bctype[j]=='patch':
            	    f.write('        type    zeroGradient;\n')
		elif bctype[j]=='symmetry':
            	    f.write('        type    symmetry;\n')
		elif bctype[j]=='symmetryPlane':
            	    f.write('        type    symmetryPlane;\n')
		elif bctype[j]=='empty':
            	    f.write('        type    empty;\n')
		elif bctype[j]=='cyclic':
            	    f.write('        type    cyclic;\n')
		elif bctype[j]=='cyclicAMI':
            	    f.write('        type    cyclicAMI;\n')
       	        f.write('    }\n')
            f.write('}\n')
	    f.close()
